/*******
 *
 * AutoCorrelation.c
 *
 * This is a simple test harness to make sure that things are working.
 * It tests the auto-correlation routine of the Analysis module.
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	12th July 1993.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 ********/

#include <stdlib.h> 
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "CRLHeaders.h"

/******************************************************************************/ 
/****************************** Constant definitions **************************/ 
/******************************************************************************/ 

#define	PARAMETERS_FILE		"AutoCorrelation.par" /* Name of paramters file. */
#define	NUM_CHANNELS		1				/* No. of Signal Channels. */
#define CHANNEL				0				/* Work channel. */

/******************************************************************************/ 
/****************************** Global variables ******************************/ 
/******************************************************************************/

char	outputFile[MAXLINE], stParFile[MAXLINE], stModuleName[MAXLINE];
double	autoCorrelationPeriod, timeConstant;


/******************************************************************************/ 
/****************************** Functions and subroutines *********************/ 
/******************************************************************************/

/****************************** ReadParsFromFile ******************************/

/*
 * This program reads a specified number of parameters from a file.
 * It expects there to be one parameter per line.
 */
 
void
ReadParsFromFile(char *fileName)
{
	char	line[MAXLINE];
	FILE	*fp;
	
	if ((fp = fopen(fileName, "r")) == NULL) {
		NotifyError("ReadTestPars: Cannot open data file '%s'.\n", fileName);
		exit(1);
	}
	printf("Reading parameters from file: %s\n", fileName);
	Init_ParFile();
	GetPars_ParFile(fp, "%s", outputFile);
	GetPars_ParFile(fp, "%s %s", stParFile, stModuleName);
	GetPars_ParFile(fp, "%lf", &autoCorrelationPeriod);
	GetPars_ParFile(fp, "%lf", &timeConstant);
	fclose(fp);
	Free_ParFile();
	
}

/******************************************************************************/ 
/****************************** Main Body *************************************/ 
/******************************************************************************/ 

int main()
{
	int		i;
	double	intensity[NUM_CHANNELS];
	EarObjectPtr	stimulus = NULL, autoCorrelation = NULL;
	
	printf("Starting Test Harness...\n");
	ReadParsFromFile(PARAMETERS_FILE);
	printf("\nThis test generates a %s and the calculated\n", stModuleName);
	printf("intensity of the signal is generated.\n");
	printf("The last %lg ms of the signal is then auto correlated.\n\n",
	  MSEC(autoCorrelationPeriod));
	
	/* Initialise EarObjects */

	if ((stimulus = Init_EarObject(stModuleName)) == NULL)
		exit(1);
	autoCorrelation = Init_EarObject("null");

	/* Set up EarObject connections. */
	
	ConnectOutSignalToIn_EarObject(stimulus, autoCorrelation);

	/* Initialise modules. */

	if (!DoFun1(ReadPars, stimulus, stParFile))
		exit(1);
	DoFun(PrintPars, stimulus);
	  
	/* Start main process and print diagonstics. */

	DoProcess(GenerateSignal, stimulus);
	PrintProcessName_EarObject("Stimulus signal generated by the '%s' "\
	  "paradigm.\n", stimulus);
	CalcIntensities_GenAnalysis(intensity, stimulus->outSignal, 0.0);
	printf("The calculated intensity = %lg dB (SPL)\n", intensity[CHANNEL]);
	
	CalcACF_ExpAnalysis(autoCorrelation, stimulus->outSignal->length *
	  stimulus->outSignal->dt, timeConstant, autoCorrelationPeriod);
	PrintProcessName_EarObject("Signal subjected to '%s'.\n",
	  autoCorrelation);
	
	WriteOutSignal_DataFile(outputFile, autoCorrelation);

	FreeAll_EarObject();

	printf("Finished test.\n");
    exit(0);
	
}
