/*******
 *
 * CalcAverage.c
 *
 * This is a simple test harness to make sure that things are working.
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	12th July 1993.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 ********/

#include <stdlib.h> 
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "CRLHeaders.h"

/******************************************************************************/ 
/****************************** Constant definitions **************************/ 
/******************************************************************************/ 

#define	PARAMETERS_FILE		"CalcAverage.par"	/* Name of paramters file. */
#define	NUM_CHANNELS		1			/* Number of signal channels. */
#define	CHANNEL				0			/* Channel for testing. */

/******************************************************************************/ 
/****************************** Global variables ******************************/ 
/******************************************************************************/

char	stParFile[MAXLINE], stModuleName[MAXLINE];
double	startAvePeriod, endAvePeriod;

/******************************************************************************/ 
/****************************** Functions and subroutines *********************/ 
/******************************************************************************/

/****************************** ReadParsFromFile ******************************/

/*
 * This program reads a specified number of parameters from a file.
 * It expects there to be one parameter per line.
 */
 
void
ReadParsFromFile(char *fileName)
{
	char	line[MAXLINE];
	FILE	*fp;
	
	if ((fp = fopen(fileName, "r")) == NULL) {
		NotifyError("ReadTestPars: Cannot open data file '%s'.\n", fileName);
		exit(1);
	}
	printf("Reading parameters from file: %s\n", fileName);
	Init_ParFile();
	GetPars_ParFile(fp, "%s %s", stParFile, stModuleName);
	GetPars_ParFile(fp, "%lf", &startAvePeriod);
	GetPars_ParFile(fp, "%lf", &endAvePeriod);
	fclose(fp);
	Free_ParFile();
	
}

/******************************************************************************/ 
/****************************** Main Body *************************************/ 
/******************************************************************************/ 

int main()
{
	double	average[NUM_CHANNELS];
	EarObjectPtr	stimulus = NULL;
	
	printf("Starting Test Harness...\n");
	ReadParsFromFile(PARAMETERS_FILE);
	printf("\nThis test generates a '%s' stimulus.\n", stModuleName);
	printf("The calculated intensity of the signal is generated.\n");
	printf("The the signal period %lg - %lg ms is averaged and the result\n",
	  MSEC(startAvePeriod), MSEC(endAvePeriod));
	printf("is printed to the screen.\n\n");
	
	/* Initialise EarObjects */

	if ((stimulus = Init_EarObject(stModuleName)) == NULL)
		exit(1);

	/* Initialise modules. */

	if (!DoFun1(ReadPars, stimulus, stParFile))
		exit(1);
	DoFun(PrintPars, stimulus);

	/* Start main process and print diagonstics. */

	DoProcess(GenerateSignal, stimulus);
	PrintProcessName_EarObject("Stimulus signal generated by the '%s' "\
	  "paradigm.\n", stimulus);
	CalcAverages_GenAnalysis(average, stimulus->outSignal, startAvePeriod,
	  endAvePeriod);
	printf("The average value for the period is %lg uPa\n", average[CHANNEL]);
	printf("Averaging over the entire signal should produce a very small\n");
	printf("(almost zero) result.\n");
	Free_EarObject(&stimulus);

	printf("Finished test.\n");
    exit(0);
	
}
