/*******
 *
 * PostStimTHistogram.c
 *
 * This is a simple test harness to make sure that things are working.
 * It tests the Post Stimulus Histogram routine.
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	12th July 1993.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 ********/

#include <stdlib.h> 
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "CRLHeaders.h"

/******************************************************************************/ 
/****************************** Constant definitions **************************/ 
/******************************************************************************/ 

#define	PARAMETERS_FILE	"PostStimTHistogram.par" /* Name of paramters file.*/

/******************************************************************************/ 
/****************************** Global variables ******************************/ 
/******************************************************************************/

char	outputFile1[MAXLINE], outputFile2[MAXLINE], outputFile3[MAXLINE];
char	stParFile[MAXLINE], stModuleName[MAXLINE];
double	stimulusFrequency, binWidth, smallerBinWidth;

/******************************************************************************/ 
/****************************** Functions and subroutines *********************/ 
/******************************************************************************/

/****************************** ReadParsFromFile ******************************/

/*
 * This program reads a specified number of parameters from a file.
 * It expects there to be one parameter per line.
 */
 
void
ReadParsFromFile(char *fileName)
{
	char	line[MAXLINE];
	FILE	*fp;
	
	if ((fp = fopen(fileName, "r")) == NULL) {
		NotifyError("ReadTestPars: Cannot open data file '%s'.\n", fileName);
		exit(1);
	}
	printf("Reading parameters from file: %s\n", fileName);
	Init_ParFile();
	GetPars_ParFile(fp, "%s", outputFile1);
	GetPars_ParFile(fp, "%s", outputFile2);
	GetPars_ParFile(fp, "%s", outputFile3);
	GetPars_ParFile(fp, "%s %s", stParFile, stModuleName);
	GetPars_ParFile(fp, "%lf", &stimulusFrequency);
	GetPars_ParFile(fp, "%lf", &binWidth);
	GetPars_ParFile(fp, "%lf", &smallerBinWidth);
	fclose(fp);
	Free_ParFile();
	
}

/******************************************************************************/ 
/****************************** Main Body *************************************/ 
/******************************************************************************/ 

int main()
{
	double	intensity[1];
	EarObjectPtr	stimulus = NULL, histogram = NULL;
	
	printf("Starting Test Harness...\n");
	
	ReadParsFromFile(PARAMETERS_FILE);
	printf("\nThis test generates a pure tone stimulus.\n");
	
	/* Initialise EarObjects. */
	
	if ((stimulus = Init_EarObject(stModuleName)) == NULL)
		exit(1);
    histogram = Init_EarObject("null");

	/* Set up EarObject connections. */
	
	ConnectOutSignalToIn_EarObject(stimulus, histogram);

	/* Initialise modules. */

	if (!DoFun1(ReadPars, stimulus, stParFile))
		exit(1);
	SetFrequency_PureTone(stimulusFrequency);
	DoFun(PrintPars, stimulus);

	/* Start main process. */

	DoProcess(GenerateSignal, stimulus);
	PrintProcessName_EarObject("Stimulus signal generated by the '%s' "\
	  "paradigm.\n", stimulus);

    printf("This first run tests the normal PSTH behaviour, the output "\
      "should be\n");
    printf("zero in all bins (approximately zero).\n\n");
	CalcPSTH_ExpAnalysis(histogram, binWidth);	
	WriteOutSignal_DataFile(outputFile1, histogram);

	printf("\nThis next step checks that the same EarObject can be used "\
	  "twice.\n");
	CalcPSTH_ExpAnalysis(histogram, smallerBinWidth);	
	WriteOutSignal_DataFile(outputFile2, histogram);

	printf("\nhis next step checks that a PSTH with bin width = Period of "\
	  "wave gives a\n");
	printf("zero-level output.\n");
	CalcPSTH_ExpAnalysis(histogram, 1.0 / stimulusFrequency);	
	WriteOutSignal_DataFile(outputFile3, histogram);

	FreeAll_EarObject();

	printf("Finished test.\n");
    exit(0);
	
}
