/*******
 *
 * RealFT.c
 *
 * This is a simple test program, producing output from a stimulus generation
 * paradigm.
 * It has been written using generic programming.
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	1st March 1994.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 ********/

#include <stdlib.h> 
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "CRLHeaders.h"

/******************************************************************************/ 
/****************************** Constant definitions **************************/ 
/******************************************************************************/ 

#define	PARAMETERS_FILE		"RealFT.par"	/* Name of paramters file.*/
#define	NUM_CHANNELS		1			/* No. of channels for the filter. */
#define CHANNEL				0			/* Channel no. for test. */

/******************************************************************************/ 
/****************************** Global variables ******************************/ 
/******************************************************************************/

char	outputFile1[MAXLINE], outputFile2[MAXLINE], stParFile[MAXLINE];
char	stModuleName[MAXLINE];

double	rampInterval;

/******************************************************************************/ 
/****************************** Functions and subroutines *********************/ 
/******************************************************************************/

/****************************** ReadParsFromFile ******************************/

/*
 * This program reads a specified number of parameters from a file.
 * It expects there to be one parameter per line.
 */
 
void
ReadParsFromFile(char *fileName)
{
	FILE	*fp;
	
	if ((fp = fopen(fileName, "r")) == NULL) {
		NotifyError("ReadTestPars: Cannot open data file '%s'.\n", fileName);
		exit(1);
	}
	printf("Reading parameters from file: %s\n", fileName);
	Init_ParFile();
	GetPars_ParFile(fp, "%s", outputFile1);
	GetPars_ParFile(fp, "%s", outputFile2);
	GetPars_ParFile(fp, "%s %s", stParFile, stModuleName);
	GetPars_ParFile(fp, "%lf", &rampInterval);
	fclose(fp);
	Free_ParFile();
	
}

/******************************************************************************/ 
/****************************** Main Body *************************************/ 
/******************************************************************************/ 

int main()
{
	double	intensity[NUM_CHANNELS];
	EarObjectPtr	stimulus = NULL;
	
	printf("Starting Test Harness...\n");
	
	ReadParsFromFile(PARAMETERS_FILE);
	printf("\nIn this test a '%s' stimulus is generated.\n", stModuleName);
	printf("A forwards, then a reverse FT is calculated.\n");
	printf("If necessary, a rise time of %g ms is applied to the stimulus.\n",
	  MSEC(rampInterval));
	printf("\n");
	
	/* Initialise EarObjects. */
	
	if ((stimulus = Init_EarObject(stModuleName)) == NULL)
		exit(1);

	/* Initialise modules. */

	printf("Module parameters...\n\n" );

	if (!DoFun1( ReadPars, stimulus, stParFile))
		exit(1);
	DoFun( PrintPars, stimulus );

	/* Start main process and print diagonstics. */
	
	printf("\nStarting main process...\n\n" );
	DoProcess( GenerateSignal, stimulus );
	PrintProcessName_EarObject("1-stimulus: '%s'.\n", stimulus );
	if (!stimulus->outSignal->rampFlag ) {
		RampUpOutSignal_Ramp(stimulus, Sine_Ramp, rampInterval );
		printf("\tRealFT: Stimulus has been ramped.\n" );
	}
	CalcIntensities_GenAnalysis(intensity, stimulus->outSignal, rampInterval );
	printf("\tStimulus intensity = %g dB SPL.\n", intensity[CHANNEL] );
	RealFT_FourierT(stimulus->outSignal, FORWARD_FT);
	WriteOutSignal_DataFile(outputFile1, stimulus );
	RealFT_FourierT(stimulus->outSignal, BACKWARD_FT);
	WriteOutSignal_DataFile(outputFile2, stimulus );

	FreeAll_EarObject();

	printf("Finished test.\n");
    exit(0);
	
}
