/*******************
 *
 * ReadFile.c
 *
 * This is a test file for the reading of sound format files.
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
 
#include "CRLHeaders.h"

/******************************************************************************/
/****************************** Constant definitions **************************/
/******************************************************************************/

#define	PARAMETERS_FILE		"ReadFile.par"	/* Name of paramters file. */

/******************************************************************************/
/****************************** Global variables ******************************/
/******************************************************************************/

char	inputFile[MAXLINE], outputFile[MAXLINE];

/******************************************************************************/
/****************************** Functions and subroutines *********************/
/******************************************************************************/

/****************************** ReadParsFromFile ******************************/

/*
 * This program reads a specified number of parameters from a file.
 * It expects there to be one parameter per line.
 */
 
void
ReadParsFromFile(char *fileName)
{
	char	line[MAXLINE];
	FILE	*fp;
	
	if ((fp = fopen(fileName, "r")) == NULL) {
		NotifyError("ReadTestPars: Cannot open data file '%s'.\n", fileName);
		exit(1);
	}
	printf("Reading parameters from file: %s\n", fileName);
	Init_ParFile();
	GetPars_ParFile(fp, "%s", inputFile);
	GetPars_ParFile(fp, "%s", outputFile);
	fclose(fp);
	Free_ParFile();
	
}

/******************************************************************************/
/****************************** Main Body *************************************/
/******************************************************************************/

int main()
{
	int		i;
	double	intensities[MAX_CHANNELS];
	EarObjectPtr	stimulus;
	
	printf("Starting Test Harness...\n");
	
	ReadParsFromFile(PARAMETERS_FILE);
	printf("\nThis routine reads data from the '%s' sound file, and writes\n",
	  inputFile);
	printf("it to the sound file '%s'.\n", outputFile);
	printf("The intensities (for the different channels) of the input signal\n");
	printf("Are calculated.\n\n");
	  
	stimulus = Init_EarObject("null");
	
	if (!ReadSignal_DataFile(inputFile, stimulus)) {
		NotifyError("ReadFile: Could not read sound data file.");
		exit(1);
	}
	PrintProcessName_EarObject("Stimulus signal read from the '%s'.\n",
	  stimulus);
	CalcIntensities_GenAnalysis(intensities, stimulus->outSignal, 0.0);
	printf("The calculated intensities are:\n\n");
	for (i = 0; i < stimulus->outSignal->numChannels; i++)
		printf("%2d: = %lg dB (SPL)\n", i, intensities[i]);
	printf("\n");
	WriteOutSignal_DataFile(outputFile, stimulus);
	Free_EarObject(&stimulus);

	printf("Finished test.\n");
    exit(0);
	
}
