/*******
 *
 * WriteFile.c
 *
 * This is a simple test harness to make sure that things are working.
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	12th July 1993.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 ********/

#include <stdlib.h> 
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "CRLHeaders.h"

/******************************************************************************/ 
/****************************** Constant definitions **************************/ 
/******************************************************************************/ 

#define	PARAMETERS_FILE		"WriteFile.par"	/* Name of paramters file. */
#define	NUM_CHANNELS		1			/* Number of signal channels. */
#define	CHANNEL				0			/* Channel for testing. */

/******************************************************************************/ 
/****************************** Global variables ******************************/ 
/******************************************************************************/

char	outputFile[MAXLINE], pTParFile[MAXLINE];

/******************************************************************************/ 
/****************************** Functions and subroutines *********************/ 
/******************************************************************************/

/****************************** ReadParsFromFile ******************************/

/*
 * This program reads a specified number of parameters from a file.
 * It expects there to be one parameter per line.
 */
 
void
ReadParsFromFile(char *fileName)
{
	char	line[MAXLINE];
	FILE	*fp;
	
	if ((fp = fopen(fileName, "r")) == NULL) {
		NotifyError("ReadTestPars: Cannot open data file '%s'.\n", fileName);
		exit(1);
	}
	printf("Reading parameters from file: %s\n", fileName);
	Init_ParFile();
	GetPars_ParFile(fp, "%s", outputFile);
	GetPars_ParFile(fp, "%s", pTParFile);
	fclose(fp);
	Free_ParFile();
	
}

/******************************************************************************/ 
/****************************** Main Body *************************************/ 
/******************************************************************************/ 

int main()
{
	int		i;
	double	intensity[NUM_CHANNELS];
	EarObjectPtr	stimulus;
	
	printf("Starting Test Harness...\n");
	
	ReadParsFromFile(PARAMETERS_FILE);
	printf("\nThis test generates a pure tone stimulus and writes it to the\n");
	printf(" file '%s'.\n", outputFile);
	printf("The calculated intensity of the signal is generated.\n");
	
	/* Initialise EarObjects. */
	
	stimulus = Init_EarObject("stimulus for WriteFile.c");

	/* Initialise modules. */

	if (!ReadPars_PureTone(pTParFile))
		exit(1);

	/* Start main process. */

	printf("\n");
	GenerateSignal_PureTone(stimulus);
	PrintProcessName_EarObject("Stimulus signal generated by the '%s' "\
	  "paradigm.\n", stimulus);
	CalcIntensities_GenAnalysis(intensity, stimulus->outSignal, 0.0);
	printf("The calculated intensity = %lg dB (SPL)\n", intensity[CHANNEL]);
	if (!WriteOutSignal_DataFile(outputFile, stimulus)) {
		NotifyError("WriteFile: Could not write file.");
		exit(1);
	}
	
	Free_PureTone();
	Free_EarObject(&stimulus);

	printf("Finished test.\n");
    exit(0);
	
}
