/**********************
 *
 * ERBCFList.c
 *
 * This program tests the GenerateERB routine from the CFList module.
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	12th July 1993.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 ***********************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "CRLHeaders.h"

/******************************************************************************/ 
/****************************** Constant definitions **************************/ 
/******************************************************************************/ 

#define	PARAMETERS_FILE		"ERBCFList.par"	/* Name of paramters file. */

/******************************************************************************/ 
/****************************** Global variables ******************************/ 
/******************************************************************************/

char	outputFile[MAXLINE];
int		numReadings;
double	testFrequency, lowestFrequency, highestFrequency, eRBDensity;

/******************************************************************************/ 
/****************************** Functions and subroutines *********************/ 
/******************************************************************************/

/****************************** ReadParsFromFile ******************************/

/*
 * This program reads a specified number of parameters from a file.
 * It expects there to be one parameter per line.
 */
 
void
ReadParsFromFile(char *fileName)
{
	char	line[MAXLINE];
	FILE	*fp;
	
	if ((fp = fopen(fileName, "r")) == NULL) {
		NotifyError("ReadTestPars: Cannot open data file '%s'.\n", fileName);
		exit(1);
	}
	printf("Reading parameters from file: %s\n", fileName);
	Init_ParFile();
	GetPars_ParFile(fp, "%s", outputFile);
	GetPars_ParFile(fp, "%lf", &testFrequency);
	GetPars_ParFile(fp, "%lf", &lowestFrequency);
	GetPars_ParFile(fp, "%lf", &highestFrequency);
	GetPars_ParFile(fp, "%lf", &eRBDensity);
	fclose(fp);
	Free_ParFile();
	
}

/******************************************************************************/ 
/****************************** Main Body *************************************/ 
/******************************************************************************/ 

int main()
{
	int		i;
	double	theERB, theERBRate;
	FILE	*fp;
	CFListPtr	theCFList = NULL;

	ReadParsFromFile(PARAMETERS_FILE);
	printf("This test program checks the operation of the CFList module.\n");
	printf("It tests the calculation of ERB, ERB rate and the respective\n");
	printf("inverse functions.\n");
	printf("A list of equally ERB-spaced centre frequencies is also\n");
	printf("calculated from %lg Hz to %lg Hz, with an ERB density of %lg.\n\n",
	  lowestFrequency, highestFrequency, eRBDensity);
	
	theERB = ERBFromF_ERB(testFrequency);
	printf("The ERB for %lg Hz is %lg Hz\n", testFrequency, theERB);
	printf("The Frequency for ERB = %lg Hz is %lg Hz\n", theERB,
	  FFromERB_ERB(theERB));
	theERBRate = ERBRateFromF_ERB(testFrequency);
	printf("The ERB rate for %lg Hz is %lg Hz\n", testFrequency, theERBRate);
	printf("The Frequency for ERB rate = %lg Hz is %lg Hz\n", theERBRate,
	  FFromERBRate_ERB(theERBRate));

	theCFList = GenerateERB_CFList(lowestFrequency, highestFrequency,
	  eRBDensity);

	if (theCFList == NULL) {
		printf("CentreFreqList: Centre frequency list not generated.n");
		exit(1);
	}
	if ((fp = fopen(outputFile, "w")) == NULL) {
		fprintf(stderr, "CentreFreqList: Cannot open %s for output file.\n",
		  outputFile);
		exit(1);
	}
	for (i = 0; i < theCFList->numChannels; i++)
		fprintf(fp, "%5d \t %8.2lf \n", i, theCFList->channel[i]);
	fclose(fp);
	printf("Centre frequency list output to file '%s'.\n", outputFile);
	free(theCFList);
	return(0);
	
}

