/**************
 *
 * ERBCalculations.c
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	12th July 1993.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 **************/
 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "CRLHeaders.h"

/******************************************************************************/ 
/****************************** Constant definitions **************************/ 
/******************************************************************************/ 

#define	PARAMETERS_FILE		"ERBCalculations.par"	/* Name of paramters file. */

/******************************************************************************/ 
/****************************** Global variables ******************************/ 
/******************************************************************************/

char	outputFile[MAXLINE];
int		numReadings;
double	initialFrequency;

/******************************************************************************/ 
/****************************** Functions and subroutines *********************/ 
/******************************************************************************/

/****************************** ReadParsFromFile ******************************/

/*
 * This program reads a specified number of parameters from a file.
 * It expects there to be one parameter per line.
 */
 
void
ReadParsFromFile(char *fileName)
{
	char	line[MAXLINE];
	FILE	*fp;
	
	if ((fp = fopen(fileName, "r")) == NULL) {
		NotifyError("ReadTestPars: Cannot open data file '%s'.\n", fileName);
		exit(1);
	}
	printf("Reading parameters from file: %s\n", fileName);
	Init_ParFile();
	GetPars_ParFile(fp, "%s", outputFile);
	GetPars_ParFile(fp, "%d", &numReadings);
	GetPars_ParFile(fp, "%lf", &initialFrequency);
	fclose(fp);
	Free_ParFile();
	
}

/******************************************************************************/ 
/****************************** Main Body *************************************/ 
/******************************************************************************/ 

int main()
{
	int		i;
	double	frequency, erb, qValue;
	FILE	*fp;
	
	printf("Starting Test Harness...\n");
	
	ReadParsFromFile(PARAMETERS_FILE);
	printf("\nThis test generates a table of frequency - ERB values.\n");
	printf("The frequency starts at %lg Hz and %d values are calculated\n",
	  initialFrequency, numReadings);
	printf("with the frequency increasing logarithmically.\n\n");
	  
	if ((fp = fopen(outputFile, "w")) == NULL ) {
		fprintf(stderr, "ErbCalculations: Could not open file.\n");
		exit(1);
	}
	for (i = 0; i < 9; i++) {
		frequency = initialFrequency * pow(2.0, (double) i);
		qValue = frequency / (0.85 * ERBFromF_ERB(frequency));
		fprintf(fp, "%10.5lf \t %10.5lf\n", frequency, qValue);
	}
	fclose(fp);
	printf("Data file output to file '%s'.\n", outputFile);
	exit(0);
}