/*******
 *
 * Ramps.c
 *
 * This is a simple test harness to make sure that things are working.
 * It tests the Linear ramp routines.
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	12th July 1993.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 ********/

#include <stdlib.h> 
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "CRLHeaders.h"

/******************************************************************************/ 
/****************************** Constant definitions **************************/ 
/******************************************************************************/ 

#define	PARAMETERS_FILE		"Ramps.par"	/* Name of paramters file. */
#define	NUM_CHANNELS		1			/* No. of channels for the filter. */
#define CHANNEL				0			/* Channel no. for test. */

/******************************************************************************/ 
/****************************** Global variables ******************************/ 
/******************************************************************************/

char	outputFile[MAXLINE], stParFile[MAXLINE], stModuleName[MAXLINE];
double	rampInterval;

/******************************************************************************/ 
/****************************** Functions and subroutines *********************/ 
/******************************************************************************/

/****************************** ReadParsFromFile ******************************/

/*
 * This program reads a specified number of parameters from a file.
 */
 
void
ReadParsFromFile(char *fileName)
{
	char	line[MAXLINE];
	FILE	*fp;
	
	if ((fp = fopen(fileName, "r")) == NULL) {
		NotifyError("ReadTestPars: Cannot open data file '%s'.\n", fileName);
		exit(1);
	}
	printf("Reading parameters from file: %s\n", fileName);
	Init_ParFile();
	GetPars_ParFile(fp, "%s", outputFile);
	GetPars_ParFile(fp, "%s %s", stParFile, stModuleName);
	GetPars_ParFile(fp, "%lf", &rampInterval);
	fclose(fp);
	Free_ParFile();
	
}

/******************************************************************************/ 
/****************************** Main Body *************************************/ 
/******************************************************************************/ 

int main()
{
	double	intensity[NUM_CHANNELS];
	EarObjectPtr	stimulus = NULL;
	
	printf("Starting Test Harness...\n");
	
	ReadParsFromFile(PARAMETERS_FILE);
	printf("\nThis test generates a %s stimulus of %lg Hz\n",
	  stModuleName);
	printf("The signal has a ramp interval of %lg ms applied to it if it \n",
	  MSEC(rampInterval));
	printf("Possible e.g. click stimuli should not be ramped.\n");
	printf("The calculated intensity of the signal is generated.\n\n");
	
	/* Initialise EarObjects. */
	
	if ((stimulus = Init_EarObject(stModuleName)) == NULL)
		exit(1);

	/* Initialise modules. */

	if (!DoFun1(ReadPars, stimulus, stParFile))
		exit(1);
	DoFun(PrintPars, stimulus);

	/* Start main process. */

	DoProcess(GenerateSignal, stimulus);
	PrintProcessName_EarObject("Stimulus signal generated by the '%s' "\
	  "paradigm.\n", stimulus);
	CalcIntensities_GenAnalysis(intensity, stimulus->outSignal, 0.0);
	printf("The calculated intensity = %lg dB (SPL)\n", intensity[CHANNEL]);
	RampUpOutSignal_Ramp(stimulus, Linear_Ramp, rampInterval);
	RampDownOutSignal_Ramp(stimulus, Sine_Ramp, rampInterval);
	WriteOutSignal_DataFile(outputFile, stimulus);
	FreeAll_EarObject();

	printf("Finished test.\n");
    exit(0);
	
}
