/******************
 *
 * FiDataFile.h
 *
 * This is the header for the Filing module deals with the writing (and later
 * reading) of data files.
 *
 ******************/

#ifndef	_FIDATAFILE_H
#define _FIDATAFILE_H	1

/******************************************************************************/
/*************************** Constant Definitions *****************************/
/******************************************************************************/

/*************************** File format definitions **************************/

#define	AIF_DATA_FILE		0
#define	AIFF_DATA_FILE		1
#define	DAT_DATA_FILE		2
#define	RAW_DATA_FILE		3
#define	WAV_DATA_FILE		4
#define	WAVE_DATA_FILE		5

/*************************** AIFF definitions *********************************/

#define	AIFF_FORM		0x464f524d		/* "FORM" */
#define	AIFF_AIFF		0x41494646		/* "AIFF" */
#define	AIFF_COMM		0x434f4d4d		/* "COMM" */
#define	AIFF_SSND		0x53534e44		/* "SSND" */

/*************************** Microsoft Wave definitions format ****************/

#define WAVE_RIFF		0x46464952		/* "FFIR" (RIFF) */
#define WAVE_WAVE		0x45564157		/* "EVAW" (WAVE) */
#define WAVE_FMT		0x20746D66		/* "fmt " (fmt ) */
#define WAVE_DATA		0x61746164		/* "atad" (data) */
#define WAVE_PCM_CODE	1
#define WAVE_MONO		1
#define WAVE_STEREO		2

/*************************** Misc definitions *********************************/

#define BIG_ENDIAN_DATA_FILE	2		/* Option for binary reading/writing */
#define LITTLE_ENDIAN_DATA_FILE	1		/* Option for binary reading/writing */

#ifndef SEEK_SET

#	define SEEK_SET		0
#	define SEEK_CUR		1
#	define SEEK_END		2

#endif

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/

typedef struct {

	/* Form AIFF chunk data (FORM, AIFF). */
	int32	chunkSize;

	/* Common Chunk data (COMM) */
	int		numChannels;		/* No. of channels - mono, stereo...etc. */
	int32	numSampleFrames;	/* No. of sample frames in sound chunk. */
	int		sampleSize;			/* No. of data bits for each data point. */
	double	sampleRate;			/* Sample rate of data in sound chunk. */

	/* Sound data Chunk data (SSND). */
	
	int32	offset;				/* - where first sample frame starts */
	int		blockSize;			/* used in conjuction with offset. */
	int		soundPosition;		/* Start of the sound data - used later. */

} AIFFParams, *AIFFParamsPtr;

typedef struct {

	int32	identifier;		/* 'RIFF' */
	int32	length;			/* total file length. */
	int32	chunkType;		/* 'WAVE' */

	int32	subChunkType;	/* 'fmt ' */
	int32	subChunkLength;	/* length of sub_chunk, =16 */
	int16	format;			/* should be 1 for PCM-code */
	int16	numChannels;	/* 1 Mono, 2 Stereo */
	int32	sampleRate;		/* frequency of sample */
	int32	byteRate;		/* No. of bytes per second. */
	int16	sampleBytes;	/* samplesize; 1 or 2 bytes */
	int16	sampleSize;		/* 8, 12 or 16 bit */ 

	int32	dataChunk;		/* 'data' */
	int32	dataChunkLength;/* sample count */
  
} WaveHeader, *WaveHeaderPtr;

typedef struct {

	BOOLEAN	mustSetMaxSamplesFlag;	/* Set when ReadRay or ReadASCII used. */
	char	name[MAXLINE];		/* - used by Generate Signal, set by ReadPars.*/
	int		wordSize;			/* can be either 1 or 2 byte words. */
	int		endian;				/* can be either 0 or 1, little- or big-endian*/
	double	defaultSampleRate;	/* - for when this is not specified. */
	int32 	maxSamples;			/* This can be set to restrict the data size. */ 

	/* Generic IO routines for reading big/little-endian data. */
	
	void	(* Write16Bits)(FILE *, int16);
	void	(* Write32Bits)(FILE *, int32);
	int16	(* Read16Bits)(FILE *);
	int32	(* Read32Bits)(FILE *);

} DataFile;


/******************************************************************************/
/*************************** External variables *******************************/
/******************************************************************************/

extern	DataFile	dataFile, *dataFilePtr;
extern int32	maxSamples;

/******************************************************************************/
/*************************** Subroutine declarations **************************/
/******************************************************************************/

BOOLEAN		CheckPars_DataFile(void);

ChanLen		FileLength_DataFile(FILE *fp);

char *		Format_DataFile(int i);

double		GetASCIISamplingInterval_DataFile(FILE *fp);

char		*GetSuffix_DataFile(char *fileName);

void		Init_DataFile(void);

int			NumberOfColumns_DataFile(FILE *fp);

void		PrintPars_DataFile(void);

void		ReadAIFFParams_DataFile(FILE *fp, AIFFParamsPtr p);

BOOLEAN		ReadAIFF_DataFile(char *fileName, EarObjectPtr data);

BOOLEAN		ReadASCII_DataFile(char *fileName, EarObjectPtr data);

BOOLEAN		ReadPars_DataFile(char *fileName);

BOOLEAN		ReadRaw_DataFile(char *fileName, EarObjectPtr data);

BOOLEAN		ReadSoundChunkData_DataFile(FILE *fp, EarObjectPtr data,
			  AIFFParamsPtr p);
			  
BOOLEAN		ReadSignal_DataFile(char *fileName, EarObjectPtr data);

BOOLEAN		ReadSignal_DataFile_Named(EarObjectPtr data);

BOOLEAN		ReadWave_DataFile(char *fileName, EarObjectPtr data);

BOOLEAN		ReadWaveHeader_DataFile(FILE *fp, WaveHeaderPtr p);

BOOLEAN		SetDefaultSampleRate_DataFile(double theDefaultSampleRate);

BOOLEAN		SetEndian_DataFile(int endian);

void		SetEndianRW_DataFile(int endian);

BOOLEAN		SetFileName_DataFile(char *fileName);

BOOLEAN		SetMaxSamples_DataFile(double theMaxSamples);

BOOLEAN		SetPars_DataFile(char *theFileName, int theWordSize, int theEndian,
			  double theDefaultSampleRate, double theMaxSamples);

BOOLEAN		SetWordSize_DataFile(int wordSize);

BOOLEAN		WriteAIFF_DataFile(char *fileName, EarObjectPtr data);

void		WriteAIFFParams_DataFile(FILE *fp, EarObjectPtr data);

BOOLEAN		WriteASCII_DataFile(char *fileName, EarObjectPtr data);

BOOLEAN		WriteOutSignal_DataFile(char *fileName, EarObjectPtr data);

BOOLEAN		WriteRaw_DataFile(char *fileName, EarObjectPtr data);

BOOLEAN		WriteSignal_DataFile(FILE *fp, SignalDataPtr signal, int wordSize);

BOOLEAN		WriteWave_DataFile(char *fileName, EarObjectPtr data);

void		WriteWaveHeader_DataFile(FILE *fp, EarObjectPtr data);

#endif
