/*******
 *
 * EarObject.h
 *
 * This header file contains the EarObject class.
 *
 * by Lowel O'Mard 18-2-93.
 *
 ********/

#ifndef	_EAROBJECT_H
#define _EAROBJECT_H	1
 
/******************************************************************************/
/*************************** Constant Definitions *****************************/
/******************************************************************************/

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/

typedef unsigned int	EarObjHandle;		/* For customer management. */

typedef struct refNode *EarObjRefPtr;	/* Pre-referencing of customer node.  */

#ifndef MODULE_PTR
#	define MODULE_PTR

	typedef struct moduleStruct  *ModulePtr;/* Pre-referencing of module.  */
	
#endif

typedef struct {

	BOOLEAN		localInSignalFlag, localOutSignalFlag;	/* So that EarObjects */
										/* only destroy their own signals. */
	BOOLEAN		updateCustomersFlag;	/* This is set when an update is */
										/* required */
	EarObjHandle	handle;			/* Reference handle for manager. */
	char			*processName;	/* The origins of the input stimulus */
	SignalData		*inSignal;		/* The original signal */
	SignalData		*outSignal;		/* The signal set by the process stages */
	EarObjRefPtr	customerList;	/* Pointer to list of customers. */
	EarObjRefPtr	supplierList;	/* Pointer to list of suppliers. */
	ModulePtr		module;			/* Module linked with process. */

} EarObject, *EarObjectPtr;

typedef struct refNode {	/* Reference node */

	EarObjectPtr	earObject;
	EarObjRefPtr	next;		/* The next customer in the list, or NULL. */
	
} EarObjRef;

/******************************************************************************/
/*************************** External Functions *******************************/
/******************************************************************************/

ModulePtr	Init_ModuleMgr(char *name);

void		Free_ModuleMgr(ModulePtr *theModule);

/******************************************************************************/
/*************************** Function Prototypes ******************************/
/******************************************************************************/

BOOLEAN		AddEarObjRef_EarObject(EarObjRefPtr *theList,	
			  EarObjectPtr theCustomer);
			
BOOLEAN		ConnectOutSignalToIn_EarObject(EarObjectPtr supplier,	
			  EarObjectPtr customer);
			
EarObjRefPtr	CreateEarObjRef_EarObject(EarObjectPtr theObject);		
			
BOOLEAN		DestroyOldOutSignal_EarObject(EarObjectPtr data, int numChannels,	
			  ChanLen length, double samplingInterval);
			
void		FreeAll_EarObject(void);	
			
void		FreeEarObjRefList_EarObject(EarObjRefPtr *theList);	
			
void		FreeEarObjRef_EarObject(EarObjRefPtr *theList, 	
			  EarObjHandle theHandle);

void		Free_EarObject(EarObjectPtr *theObject);	
			
BOOLEAN		InitOutFromInSignal_EarObject(EarObjectPtr data, int numChannels);	
			
BOOLEAN		InitOutSignal_EarObject(EarObjectPtr data, int numChannels, 	
			  ChanLen length, double samplingInterval);
			
EarObjectPtr	Init_EarObject(char *moduleName);		
			
void		PrintProcessName_EarObject(char *message, EarObjectPtr data);	
			
void		RemoveEarObjRefs_EarObject(EarObjectPtr theObject);	
			
void		SetProcessName_EarObject(EarObjectPtr theObject, char *format, ...);	
			
void		UpdateCustomers_EarObject(EarObjectPtr theObject);	
			
#endif
