/*******
 *
 * LUTEar.h
 *
 * This header file contains the main system constants, needed everywhere.
 *
 * by Lowel O'Mard 18-2-93.
 *
 ********/

#ifndef	_LUTEAR_H
#define _LUTEAR_H	1

/******************************************************************************/
/*************************** Constant Definitions *****************************/
/******************************************************************************/
 
#ifndef PI
# define	PI	3.141592653589792434		/* From MacEar */
#endif
#define PIx2	6.283185307179584868
#define	SQRT_2	1.414213562

#define MAXLINE				80		/* Max. No. of characters in a line. */
#define MAX_CHANNELS		180		/* The number of channels is defined at
									 * compile time to improve the performance
									 * of this system. Referencing multiple
									 * channels from a single array would take
									 * up too much time. */
#ifndef FALSE
#	define	FALSE		0
#	define	TRUE		0xFFFF
#endif
#define	LARGE_VALUE		1.0e40		/* An abitrary large value.  HUGE_VAL is
									 * not used because gcc does not like it. */

/******************************************************************************/
/*************************** Macro definitions ********************************/
/******************************************************************************/ 

#define RMS_AMP(DBVALUE)		(20.0 * pow(10.0, (DBVALUE) / 20.0))

#define DB_SPL(RMS_AMPLITUDE)	(20.0 * log10((RMS_AMPLITUDE) / 20.0))

#define MSEC(SECONDS)			((SECONDS) * 1000.0)

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/ 

#ifndef	BOOLEAN 
#	define BOOLEAN_ALGEBRA
	typedef	int		BOOLEAN;
#endif

typedef	unsigned long	ChanLen;	/* For the channel indices. */
typedef	double			ChanData;	/* Data type for channel data. */

/******************************************************************************/
/*************************** External Variables *******************************/
/******************************************************************************/

extern FILE	*warningsFile;		/* File to which warnings should be sent. */
extern FILE	*errorsFile;		/* File to which errors should be sent. */

/******************************************************************************/
/*************************** Global Subroutines *******************************/
/******************************************************************************/

void	NotifyError(char *format, ...);

void	NotifyWarning(char *format, ...);

void	ReadParsFromFile(char *fileName);		/* Used in test programs. */

BOOLEAN	SetErrorsFile(char *fileName);

BOOLEAN	SetWarningsFile(char *fileName);

#endif
