/***********************
 *
 * ModuleMgr.c
 *
 * This module contains the module managment routines.
 * These routines carry out various functions such as freeing
 * all module memory space, and so on.
 *
 * 29-3-93 L. P. O'Mard
 *
 ***********************/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "LUTEar.h"
#include "SignalData.h"
#include "EarObject.h"

#include "UtFilters.h"
#include "UtUIEEEFloat.h"
#include "UtUPortableIO.h"
#include "UtERB.h"
#include "UtCFList.h"

#include "FiParFile.h"
#include "FiDataFile.h"

#include "Mo2HairCell.h"
#include "Mo3BasilarM.h"
#include "Mo4BasilarM.h"
#include "MoBasilarM.h"
#include "MoDendrite.h"
#include "MoHairCell.h"
#include "MoArleKim.h"
#include "MoMcGregor.h"
#include "MoPreEmph.h"
#include "MoRecptPotn.h"
#include "MoSpikeGen.h"

#include "StAMTone.h"
#include "StBPTone.h"
#include "StClick.h"
#include "StMPTone.h"
#include "StMPPTone.h"
#include "StPTone.h"
#include "StPTone2.h"
#include "StStepFun.h"

#include "ModuleMgr.h"

/******************************************************************************/
/************************** Global Variables **********************************/
/******************************************************************************/

ModuleRefPtr	moduleList = NULL;

/******************************************************************************/
/************************** Subroutines and functions *************************/
/******************************************************************************/

/************************** IdentifyModuleName ********************************/

/*
 * This routine identifies a module name from its list, returning the
 * appropriate number, or zero if it does not find identify the name.
 */

ModuleSpecifier
IdentifyModuleName_ModuleMgr(char *name)
{
	char		*p;
	int			i;
	static struct {
	
		char *name;
		ModuleSpecifier specifier;
		
	} module[] = {
	
		{ " AMTONE", AMTONE },
		{ "AUDITORYN_SPIKEGEN", AUDITORYN_SPIKEGEN },
		{ "BASILARM_GAMMAT", BASILARM_GAMMAT },
		{ "BASILARM_STONE3", BASILARM_STONE3 },
		{ "BASILARM_STONE4", BASILARM_STONE4 },
		{ "CLICK", CLICK},
		{ "DATAFILE", DATAFILE},
		{ "IHC_MEDDIS86", IHC_MEDDIS86 },
		{ "IHC_MEDDIS93", IHC_MEDDIS93 },
		{ "IHC_RECPTPOTN", IHC_RECPTPOTN },
		{ "NEURON_DENDRITELP", NEURON_DENDRITELP },
		{ "NEURON_MCGREGOR", NEURON_MCGREGOR },
		{ "NEURON_ARLEKIM", NEURON_ARLEKIM },
		{ "NULL", NULL_MODULE },
		{ "PREEMPHASIS_BANDPASS", PREEMPHASIS_BANDPASS},
		{ "PURETONE_BINAURAL", PURETONE_BINAURAL },
		{ "PURETONE_MULTI", PURETONE_MULTI },
		{ "PURETONE_MULTIPULSE", PURETONE_MULTIPULSE },
		{ "PURETONE", PURETONE },
		{ "PURETONE_2", PURETONE_2 },
		{ "STEPFUN", STEPFUN},
		{ "",  NULL_SPECIFIER}
	
	};
	
	for (p = name; *p != '\0'; p++) /* Convert to upper case */
		*p = toupper(*p);		
	for (i = 0; strcmp(name, module[i].name) != 0 && module[i].name[0] !=
	  '\0'; i++);
	return(module[i].specifier);

}

/************************** NoFunction ****************************************/

/*
 * This routine is set as the default for all non-assigned functions.
 * It will print a message it an attempt is made to use it.
 */

void *
NoFunction_ModuleMgr(void)
{
	BOOLEAN	dummy = FALSE;

	NotifyError("NoFunction_ModuleMgr: Attempted to use a "\
	  "function which does not\nexist for module.");
	return((BOOLEAN *) dummy);

}

/************************** NullFunction **************************************/

/*
 * This routine is set as the default for all non-assigned functions.
 * It will print a message it an attempt is made to use it.
 */

void *
NullFunction_ModuleMgr(void)
{
	BOOLEAN	dummy = TRUE;
	
	return((BOOLEAN *) dummy);

}

/************************** SetDefault ****************************************/

/*
 * This routine sets all of the function pointers to the default function.
 */

void
SetDefault_ModuleMgr(ModulePtr module, void *(* DefaultFunc)(void))
{
	module->CheckData = (BOOLEAN (*)(EarObjectPtr)) DefaultFunc;
	module->CheckPars = (BOOLEAN (*)(void)) DefaultFunc;
	module->GenerateSignal = (BOOLEAN (*)(EarObjectPtr)) DefaultFunc;
	module->GetCF = (double (*)(int)) DefaultFunc;
	module->GetIndividualFreq = (double (*)(int)) DefaultFunc;
	module->GetIndividualIntensity = (double (*)(int)) DefaultFunc;
	module->GetPotentialResponse = (double (*)(double)) DefaultFunc;
	module->PrintPars = (void (*)(void)) DefaultFunc;
	module->ReadPars = (BOOLEAN (*)(char *)) DefaultFunc;
	module->ReadSignal = (BOOLEAN (*)(char *, EarObjectPtr)) DefaultFunc;
	module->RunModel = (BOOLEAN (*)(EarObjectPtr)) DefaultFunc;
	module->SetAccomConst = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetActionPotential = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetAmplitude = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetBandwidths = (BOOLEAN (*)(char *, double *, size_t)) DefaultFunc;
	module->SetBaseWideFAtten = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetBeginSilence = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetBMTimeConstTm = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetBRestingCond = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetBReversalPoten = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetCarrierFrequency = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetCellRestingPot = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetCentreFreq = (BOOLEAN (*)(double *, size_t)) DefaultFunc;
	module->SetCFList = (BOOLEAN (*)(CFListPtr)) DefaultFunc;
	module->SetClickTime = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetCompressionExponent = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetCompressionThreshold = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetCutOffFrequency = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetDelayedRectKCond = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetDuration = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetEndSilence = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetFiringRate = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetFrequencies = (BOOLEAN (*)(double *)) DefaultFunc;
	module->SetFrequency = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetIndividualFreq = (BOOLEAN (*)(int, double)) DefaultFunc;
	module->SetIndividualIntensity = (BOOLEAN (*)(int, double)) DefaultFunc;
	module->SetIndividualPhase =(BOOLEAN (*)(int, double)) DefaultFunc;
	module->SetIntensities = (BOOLEAN (*)(double *)) DefaultFunc;
	module->SetIntensity = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetKDecayTConst = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetKEquilibriumPot = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetKRestingCond = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetKReversalPoten = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetLossRate = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetLowerCutOffFreq = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetMaxFreePool = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetMembraneTConst = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetModulationFrequency = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetNarrowBroadeningCoeff = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetNonLinearVConst = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetNumChannels = (BOOLEAN (*)(int)) DefaultFunc;
	module->SetNumPTones = (BOOLEAN (*)(int)) DefaultFunc;
	module->SetNumPulses = (BOOLEAN (*)(int)) DefaultFunc;
	module->SetOrder = (BOOLEAN (*)(int)) DefaultFunc;
	module->SetParamQ = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetParamV = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetPercentAM = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetPermConstA = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetPermConstB = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetPhases = (BOOLEAN (*)(double *)) DefaultFunc;
	module->SetPhaseDifference = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetPulseDuration = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetRanSeed = (BOOLEAN (*)(long)) DefaultFunc;
	module->SetRecoveryRate = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetRefractoryPeriod = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetReleaseRate = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetReplenishRate = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetRepetitionPeriod = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetReprocessRate = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetRestingThreshold = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetSamplingInterval = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetSignalMultiplier = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetThresholdTConst = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetUpperCutOffFreq = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideAttenuation = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideBroadeningCoeff = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideFAttenScale = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideFCF = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideFCentreFreq =  (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideFWidth =  (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideHighFCutOff =  (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideLowFCutOff = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideFLPCutOffScale = (BOOLEAN (*)(double)) DefaultFunc;
	module->SetWideFRefCF =  (BOOLEAN (*)(double)) DefaultFunc;
	module->WriteOutSignal = WriteOutSignal_DataFile;

}

/************************** SetAuditoryN_SpikeGen *****************************/

/*
 * This routine sets the functions for the auditory nerve spike generation
 * model module.
 */

void
SetAuditoryN_SpikeGen_ModuleMgr(ModulePtr theModule)
{
	theModule->PrintPars = PrintPars_AuditoryN_SpikeGen;
	theModule->CheckPars = CheckPars_AuditoryN_SpikeGen;
	theModule->ReadPars = ReadPars_AuditoryN_SpikeGen;
	theModule->RunModel = RunModel_AuditoryN_SpikeGen;
	theModule->SetRanSeed = SetRanSeed_AuditoryN_SpikeGen;
	theModule->SetRefractoryPeriod = SetRefractoryPeriod_AuditoryN_SpikeGen;

}

/************************** SetNeuron_DendriteLP ******************************/

/*
 * This routine sets the functions for the auditory nerve spike generation
 * model module.
 */

void
SetNeuron_DendriteLP_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_Neuron_DendriteLP;
	theModule->PrintPars = PrintPars_Neuron_DendriteLP;
	theModule->ReadPars = ReadPars_Neuron_DendriteLP;
	theModule->RunModel = RunModel_Neuron_DendriteLP;
	theModule->SetCutOffFrequency  = SetCutOffFrequency_Neuron_DendriteLP;
	theModule->SetSignalMultiplier = SetSignalMultiplier_Neuron_DendriteLP;

}

/************************** SetNeuron_ArleKim *********************************/

/*
 * This routine sets the functions for the Arle & Kim neural cell
 * model module.
 */

void
SetNeuron_ArleKim_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_Neuron_ArleKim;
	theModule->GetPotentialResponse = GetPotentialResponse_Neuron_ArleKim;
	theModule->PrintPars = PrintPars_Neuron_ArleKim;
	theModule->ReadPars = ReadPars_Neuron_ArleKim;
	theModule->RunModel = RunModel_Neuron_ArleKim;
	theModule->SetAccomConst = SetAccomConst_Neuron_ArleKim;
	theModule->SetActionPotential = SetActionPotential_Neuron_ArleKim;
	theModule->SetBRestingCond = SetBRestingCond_Neuron_ArleKim;
	theModule->SetBReversalPoten = SetBReversalPoten_Neuron_ArleKim;
	theModule->SetDelayedRectKCond = SetDelayedRectKCond_Neuron_ArleKim;
	theModule->SetKDecayTConst = SetKDecayTConst_Neuron_ArleKim;
	theModule->SetKRestingCond = SetKRestingCond_Neuron_ArleKim;
	theModule->SetKReversalPoten = SetKReversalPoten_Neuron_ArleKim;
	theModule->SetMembraneTConst = SetMembraneTConst_Neuron_ArleKim;
	theModule->SetNonLinearVConst = SetNonLinearVConst_Neuron_ArleKim;
	theModule->SetRestingThreshold = SetRestingThreshold_Neuron_ArleKim;
	theModule->SetThresholdTConst = SetThresholdTConst_Neuron_ArleKim;

}

/************************** SetNeuron_McGregor ********************************/

/*
 * This routine sets the functions for the McGregor 3 neural cell
 * model module.
 */

void
SetNeuron_McGregor_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_Neuron_McGregor;
	theModule->GetPotentialResponse = GetPotentialResponse_Neuron_McGregor;
	theModule->PrintPars = PrintPars_Neuron_McGregor;
	theModule->ReadPars = ReadPars_Neuron_McGregor;
	theModule->RunModel = RunModel_Neuron_McGregor;
	theModule->SetAccomConst = SetAccomConst_Neuron_McGregor;
	theModule->SetActionPotential = SetActionPotential_Neuron_McGregor;
	theModule->SetCellRestingPot = SetCellRestingPot_Neuron_McGregor;
	theModule->SetDelayedRectKCond = SetDelayedRectKCond_Neuron_McGregor;
	theModule->SetKDecayTConst = SetKDecayTConst_Neuron_McGregor;
	theModule->SetKEquilibriumPot = SetKEquilibriumPot_Neuron_McGregor;
	theModule->SetMembraneTConst = SetMembraneTConst_Neuron_McGregor;
	theModule->SetRestingThreshold = SetRestingThreshold_Neuron_McGregor;
	theModule->SetThresholdTConst = SetThresholdTConst_Neuron_McGregor;

}

/************************** SetPreEmphasis_BandPass ***************************/

/*
 * This routine sets the functions for the outer-/middle-ear
 * model module.
 */

void
SetPreEmphasis_BandPass_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_PreEmphasis_BandPass;
	theModule->PrintPars = PrintPars_PreEmphasis_BandPass;
	theModule->ReadPars = ReadPars_PreEmphasis_BandPass;
	theModule->RunModel = RunModel_PreEmphasis_BandPass;
	theModule->SetLowerCutOffFreq = SetLowerCutOffFreq_PreEmphasis_BandPass;
	theModule->SetUpperCutOffFreq = SetUpperCutOffFreq_PreEmphasis_BandPass;

}

/************************** SetPureTone ***************************************/

/*
 * This routine sets the functions for the pure tone signal generation module.
 */

void
SetPureTone_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_PureTone;
	theModule->GenerateSignal = GenerateSignal_PureTone;
	theModule->PrintPars = PrintPars_PureTone;
	theModule->ReadPars = ReadPars_PureTone;
	theModule->SetDuration = SetDuration_PureTone;
	theModule->SetFrequency = SetFrequency_PureTone;
	theModule->SetIntensity = SetIntensity_PureTone;
	theModule->SetSamplingInterval = SetSamplingInterval_PureTone;

}

/************************** SetClick ******************************************/

/*
 * This routine sets the functions for the click signal generation module.
 */

void
SetClick_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_Click;
	theModule->GenerateSignal = GenerateSignal_Click;
	theModule->PrintPars = PrintPars_Click;
	theModule->ReadPars = ReadPars_Click;
	theModule->SetClickTime = SetClickTime_Click;
	theModule->SetDuration = SetDuration_Click;
	theModule->SetIntensity = SetIntensity_Click;
	theModule->SetSamplingInterval = SetSamplingInterval_Click;

}

/************************** SetBPureTone **************************************/

/*
 * This routine sets the functions for the binuaural pure tone signal generation
 * module.
 */

void
SetBPureTone_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_PureTone_Binaural;
	theModule->GenerateSignal = GenerateSignal_PureTone_Binaural;
	theModule->PrintPars = PrintPars_PureTone_Binaural;
	theModule->ReadPars = ReadPars_PureTone_Binaural;
	theModule->SetDuration = SetDuration_PureTone_Binaural;
	theModule->SetIndividualFreq = SetIndividualFreq_PureTone_Binaural;
	theModule->SetIndividualIntensity = SetIndividualIntensity_PureTone_Binaural;
	theModule->SetPhaseDifference = SetPhaseDifference_PureTone_Binaural;
	theModule->SetSamplingInterval = SetSamplingInterval_PureTone_Binaural;

}

/************************** SetMPureTone **************************************/

/*
 * This routine sets the functions for the multiple pure tone signal generation
 * module.
 */

void
SetMPureTone_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_PureTone_Multi;
	theModule->GenerateSignal = GenerateSignal_PureTone_Multi;
	theModule->GetIndividualFreq = GetIndividualFreq_PureTone_Multi;
	theModule->GetIndividualIntensity = GetIndividualIntensity_PureTone_Multi;
	theModule->PrintPars = PrintPars_PureTone_Multi;
	theModule->ReadPars = ReadPars_PureTone_Multi;
	theModule->SetDuration = SetDuration_PureTone_Multi;
	theModule->SetFrequencies = SetFrequencies_PureTone_Multi;
	theModule->SetIndividualFreq = SetIndividualFreq_PureTone_Multi;
	theModule->SetIndividualIntensity = SetIndividualIntensity_PureTone_Multi;
	theModule->SetIndividualPhase =SetIndividualPhase_PureTone_Multi;
	theModule->SetIntensities = SetIntensities_PureTone_Multi;
	theModule->SetNumPTones = SetNumPTones_PureTone_Multi;
	theModule->SetPhases = SetPhases_PureTone_Multi;
	theModule->SetSamplingInterval = SetSamplingInterval_PureTone_Multi;

}

/************************** SetAMTone *****************************************/

/*
 * This routine sets the functions for the amplitude modulated pure tone signal
 * generation module.
 */

void
SetAMTone_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_AMTone;
	theModule->GenerateSignal = GenerateSignal_AMTone;
	theModule->PrintPars = PrintPars_AMTone;
	theModule->ReadPars = ReadPars_AMTone;
	theModule->SetCarrierFrequency = SetCarrierFrequency_AMTone;
	theModule->SetDuration = SetDuration_AMTone;
	theModule->SetIntensity = SetIntensity_AMTone;
	theModule->SetModulationFrequency = SetModulationFrequency_AMTone;
	theModule->SetPercentAM = SetPercentAM_AMTone;
	theModule->SetSamplingInterval = SetSamplingInterval_AMTone; 

}

/************************** SetPureTone2 **************************************/

/*
 * This routine sets the functions for the pure tone 2 signal generation
 * module - a pure tone surrounded by silence..
 */

void
SetPureTone2_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_PureTone_2;
	theModule->GenerateSignal = GenerateSignal_PureTone_2;
	theModule->PrintPars = PrintPars_PureTone_2;
	theModule->ReadPars = ReadPars_PureTone_2;
	theModule->SetBeginSilence = SetBeginSilence_PureTone_2;
	theModule->SetDuration = SetDuration_PureTone_2;
	theModule->SetEndSilence = SetEndSilence_PureTone_2;
	theModule->SetFrequency = SetFrequency_PureTone_2;
	theModule->SetIntensity = SetIntensity_PureTone_2;
	theModule->SetSamplingInterval = SetSamplingInterval_PureTone_2;

}

/************************** SetMPPureTone *************************************/

/*
 * This routine sets the functions for the multi-frequency pulsed pure tone
 * signal generation module.
 */

void
SetMPPureTone_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_PureTone_MultiPulse;
	theModule->GenerateSignal = GenerateSignal_PureTone_MultiPulse;
	theModule->GetIndividualFreq = GetIndividualFreq_PureTone_MultiPulse;
	theModule->PrintPars = PrintPars_PureTone_MultiPulse;
	theModule->ReadPars = ReadPars_PureTone_MultiPulse;
	theModule->SetBeginSilence = SetBeginSilence_PureTone_MultiPulse;
	theModule->SetDuration = SetDuration_PureTone_MultiPulse;
	theModule->SetPulseDuration = SetPulseDuration_PureTone_MultiPulse;
	theModule->SetRepetitionPeriod = SetRepetitionPeriod_PureTone_MultiPulse;
	theModule->SetFrequencies = SetFrequencies_PureTone_MultiPulse;
	theModule->SetIndividualFreq = SetIndividualFreq_PureTone_MultiPulse;
	theModule->SetNumPulses = SetNumPulses_PureTone_MultiPulse;
	theModule->SetIntensity = SetIntensity_PureTone_MultiPulse;
	theModule->SetSamplingInterval = SetSamplingInterval_PureTone_MultiPulse;

}

/************************** SetStepFun ****************************************/

/*
 * This routine sets the functions for the step function signal generation
 * module.
 */

void
SetStepFun_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_StepFunction;
	theModule->GenerateSignal = GenerateSignal_StepFunction;
	theModule->PrintPars = PrintPars_StepFunction;
	theModule->ReadPars = ReadPars_StepFunction;
	theModule->SetAmplitude = SetAmplitude_StepFunction;
	theModule->SetBeginSilence = SetBeginSilence_StepFunction;
	theModule->SetDuration = SetDuration_StepFunction;
	theModule->SetEndSilence = SetEndSilence_StepFunction;
	theModule->SetSamplingInterval = SetSamplingInterval_StepFunction;

}

/************************** SetIHC_RecptPotn **********************************/

/*
 * This routine sets the functions for the inner hair cell receptor potential
 * model module.
 */

void
SetIHC_RecptPotn_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_IHC_RecptPotn;
	theModule->PrintPars = PrintPars_IHC_RecptPotn;
	theModule->ReadPars = ReadPars_IHC_RecptPotn;
	theModule->RunModel = RunModel_IHC_RecptPotn;
	theModule->SetBMTimeConstTm = SetBMTimeConstTm_IHC_RecptPotn;
	theModule->SetParamQ = SetParamQ_IHC_RecptPotn;
	theModule->SetParamV = SetParamV_IHC_RecptPotn;

}

/************************** SetBasilarM_GammaT ********************************/

/*
 * This routine sets the functions for the gamma tone basilar membrane filter
 * model module.
 */

void
SetBasilarM_GammaT_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_BasilarM_GammaT;
	theModule->GetCF = GetCF_BasilarM_GammaT;
	theModule->PrintPars = PrintPars_BasilarM_GammaT;
	theModule->ReadPars = ReadPars_BasilarM_GammaT;
	theModule->RunModel = RunModel_BasilarM_GammaT;
	theModule->SetCentreFreq = SetCentreFreq_BasilarM_GammaT;
	theModule->SetCFList = SetCFList_BasilarM_GammaT;
	theModule->SetNumChannels = SetNumChannels_BasilarM_GammaT;
	theModule->SetOrder = SetOrder_BasilarM_GammaT;

}

/************************** SetBasilarM_Stone3 ********************************/

/*
 * This routine sets the functions for the Stone3 basilar membrane filter model
 * module.
 */

void
SetBasilarM_Stone3_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_BasilarM_Stone3;
	theModule->GetCF = GetCF_BasilarM_Stone3;
	theModule->PrintPars = PrintPars_BasilarM_Stone3;
	theModule->ReadPars = ReadPars_BasilarM_Stone3;
	theModule->RunModel = RunModel_BasilarM_Stone3;
	theModule->SetBaseWideFAtten = SetBaseWideFAtten_BasilarM_Stone3;
	theModule->SetCentreFreq = SetCentreFreq_BasilarM_Stone3;
	theModule->SetCFList = SetCFList_BasilarM_Stone3;
	theModule->SetCompressionExponent = SetCompressionExponent_BasilarM_Stone3;
	theModule->SetCompressionThreshold =
	  SetCompressionThreshold_BasilarM_Stone3;
	theModule->SetNumChannels = SetNumChannels_BasilarM_Stone3;
	theModule->SetNarrowBroadeningCoeff =
	  SetNarrowBroadeningCoeff_BasilarM_Stone3;
	theModule->SetNarrowFLPCutOffScale =
	  SetNarrowFLPCutOffScale_BasilarM_Stone3;
	theModule->SetOrder = SetOrder_BasilarM_Stone3;
	theModule->SetWideBroadeningCoeff = SetWideBroadeningCoeff_BasilarM_Stone3;
	theModule->SetWideFAttenScale = SetWideFAttenScale_BasilarM_Stone3;
	theModule->SetWideFCF = SetWideFCF_BasilarM_Stone3;
	theModule->SetWideFLPCutOffScale = SetWideFLPCutOffScale_BasilarM_Stone3;
	theModule->SetWideFRefCF = SetWideFRefCF_BasilarM_Stone3;

}

/************************** SetBasilarM_Stone4 ********************************/

/*
 * This routine sets the functions for the Stone4 basilar membrane filter model
 * module.
 */

void
SetBasilarM_Stone4_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_BasilarM_Stone4;
	theModule->GetCF = GetCF_BasilarM_Stone4;
	theModule->PrintPars = PrintPars_BasilarM_Stone4;
	theModule->ReadPars = ReadPars_BasilarM_Stone4;
	theModule->RunModel = RunModel_BasilarM_Stone4;
	theModule->SetCentreFreq = SetCentreFreq_BasilarM_Stone4;
	theModule->SetCFList = SetCFList_BasilarM_Stone4;
	theModule->SetCompressionExponent = SetCompressionExponent_BasilarM_Stone4;
	theModule->SetCompressionThreshold =
	  SetCompressionThreshold_BasilarM_Stone4;
	theModule->SetNumChannels = SetNumChannels_BasilarM_Stone4;
	theModule->SetNarrowFLPCutOffScale =
	  SetNarrowFLPCutOffScale_BasilarM_Stone4;
	theModule->SetOrder = SetOrder_BasilarM_Stone4;
	theModule->SetWideFCF = SetWideFCF_BasilarM_Stone4;
	theModule->SetWideFLPCutOffScale = SetWideFLPCutOffScale_BasilarM_Stone4;

}

/************************** SetIHC_Meddis93 ***********************************/

/*
 * This routine sets the functions for the Meddis 93 inner hair cell model
 * module.
 */

void
SetIHC_Meddis93_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckData = CheckData_IHC_Meddis93;
	theModule->CheckPars = CheckPars_IHC_Meddis93;
	theModule->PrintPars = PrintPars_IHC_Meddis93;
	theModule->ReadPars = ReadPars_IHC_Meddis93;
	theModule->RunModel = RunModel_IHC_Meddis93;
	theModule->SetLossRate = SetLossRate_IHC_Meddis93;
	theModule->SetMaxFreePool = SetMaxFreePool_IHC_Meddis93;
	theModule->SetPermeabilityPH = SetPermeabilityPH_IHC_Meddis93;
	theModule->SetPermeabilityPZ = SetPermeabilityPZ_IHC_Meddis93;
	theModule->SetReplenishRate = SetReplenishRate_IHC_Meddis93;
	theModule->SetReprocessRate = SetReprocessRate_IHC_Meddis93;
	theModule->SetRecoveryRate = SetRecoveryRate_IHC_Meddis93;

}

/************************** SetIHC_Meddis86 ***********************************/

/*
 * This routine sets the functions for the Meddis 86 inner hair cell model
 * module.
 */

void
SetIHC_Meddis86_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckData = CheckData_IHC_Meddis86;
	theModule->CheckPars = CheckPars_IHC_Meddis86;
	theModule->PrintPars = PrintPars_IHC_Meddis86;
	theModule->ReadPars = ReadPars_IHC_Meddis86;
	theModule->RunModel = RunModel_IHC_Meddis86;
	theModule->SetLossRate = SetLossRate_IHC_Meddis86;
	theModule->SetMaxFreePool = SetMaxFreePool_IHC_Meddis86;
	theModule->SetPermConstA = SetPermConstA_IHC_Meddis86;
	theModule->SetPermConstB = SetPermConstB_IHC_Meddis86;
	theModule->SetReplenishRate = SetReplenishRate_IHC_Meddis86;
	theModule->SetReprocessRate = SetReprocessRate_IHC_Meddis86;
	theModule->SetRecoveryRate = SetRecoveryRate_IHC_Meddis86;

}

/************************** SetDataFile ***************************************/

/*
 * This routine sets the functions for the DataFile module.
 * The DataFile module reads sound data formats from file.
 */

void
SetDataFile_ModuleMgr(ModulePtr theModule)
{
	theModule->CheckPars = CheckPars_DataFile;
	theModule->PrintPars = PrintPars_DataFile;
	theModule->ReadPars = ReadPars_DataFile;
	theModule->GenerateSignal = ReadSignal_DataFile_Named;
	theModule->ReadSignal = ReadSignal_DataFile;

}

/************************** Init **********************************************/

/*
 * This function initialises and sets a module structure.
 * It allocates the necessary memory for the structure and connects the
 * necessary routines.
 */

ModulePtr
Init_ModuleMgr(char *theModuleName)
{
	static ModuleHandle handleNumber = 0;
	ModulePtr	theModule;

	if ((theModule = (ModulePtr) (malloc(sizeof (Module)))) == NULL) {
		NotifyError("Init_ModuleMgr: Could not allocate memory.");
		return(NULL);
	}
	sprintf(theModule->name, "%s", theModuleName);
	SetDefault_ModuleMgr(theModule, NoFunction_ModuleMgr);
	switch (IdentifyModuleName_ModuleMgr(theModule->name)) {
	case	AMTONE:
		SetAMTone_ModuleMgr(theModule);
		break;
	case	AUDITORYN_SPIKEGEN:
		SetAuditoryN_SpikeGen_ModuleMgr(theModule);
		break;
	case	BASILARM_GAMMAT:
		SetBasilarM_GammaT_ModuleMgr(theModule);
		break;
	case	BASILARM_STONE3:
		SetBasilarM_Stone3_ModuleMgr(theModule);
		break;
	case	BASILARM_STONE4:
		SetBasilarM_Stone4_ModuleMgr(theModule);
		break;
	case	CLICK:
		SetClick_ModuleMgr(theModule);
		break;
	case	DATAFILE:
		SetDataFile_ModuleMgr(theModule);
		break;
	case	IHC_MEDDIS86:
		SetIHC_Meddis86_ModuleMgr(theModule);
		break;
	case	IHC_MEDDIS93:
		SetIHC_Meddis93_ModuleMgr(theModule);
		break;
	case	IHC_RECPTPOTN:
		SetIHC_RecptPotn_ModuleMgr(theModule);
		break;
	case	NEURON_DENDRITELP:
		SetNeuron_DendriteLP_ModuleMgr(theModule);
		break;
	case	NEURON_MCGREGOR:
		SetNeuron_McGregor_ModuleMgr(theModule);
		break;
	case	NEURON_ARLEKIM:
		SetNeuron_ArleKim_ModuleMgr(theModule);
		break;
	case	NULL_MODULE:
		SetDefault_ModuleMgr(theModule, NullFunction_ModuleMgr);
		theModule->RunModel = RunModel_ModuleMgr_Null;
		break;
	case	PREEMPHASIS_BANDPASS:
		SetPreEmphasis_BandPass_ModuleMgr(theModule);
		break;
	case	PURETONE_BINAURAL:
		SetBPureTone_ModuleMgr(theModule);
		break;
	case	PURETONE_MULTI:
		SetMPureTone_ModuleMgr(theModule);
		break;
	case	PURETONE_MULTIPULSE:
		SetMPPureTone_ModuleMgr(theModule);
		break;
	case	PURETONE:
		SetPureTone_ModuleMgr(theModule);
		break;
	case	PURETONE_2:
		SetPureTone2_ModuleMgr(theModule);
		break;
	case	STEPFUN:
		SetStepFun_ModuleMgr(theModule);
		break;
	default:
		NotifyError("Init_ModuleMgr: Unknown module '%s'.", theModuleName);
		return(NULL);
		break;
	} /* Switch */
	theModule->handle = handleNumber++;		/* Unique handle for each module. */
	if (!AddModuleRef_ModuleMgr(&moduleList, theModule)) {
		NotifyError("Init_ModuleMgr: Could not register new module.");
		Free_ModuleMgr(&theModule);
		return(NULL);
	}
	return(theModule);

}

/**************************** CreateModuleRef *********************************/

/*
 * This function returns a pointer to a module node.
 * It returns NULL if it fails.
 */
 
ModuleRefPtr
CreateModuleRef_ModuleMgr(ModulePtr theModule)
{
	ModuleRef	*newNode;
	
	if ((newNode = (ModuleRef *) malloc(sizeof (ModuleRef))) == NULL) {
		NotifyError("CreateModuleRef_ModuleMgr: Out of memory.");
		return(NULL);
	}
	newNode->module = theModule;
	newNode->next = NULL;
	return(newNode);

}

/**************************** AddModuleRef ************************************/

/*
 * This routine adds a module reference to a list.
 * It places the module handles in ascending order.
 */

BOOLEAN
AddModuleRef_ModuleMgr(ModuleRefPtr *theList, ModulePtr theModule)
{
	ModuleRefPtr	temp, p;

    if (*theList == NULL) {
    	if ((*theList = CreateModuleRef_ModuleMgr(theModule)) ==
    	  NULL) {
    		NotifyError("AddModuleRef_ModuleMgr: Could not create new node.");
    		return(FALSE);
    	}
    	
		return(TRUE);
	}
	for (p = *theList; (p->next != NULL) && (p->module->handle <
	  theModule->handle); p = p->next);
	if (p->module->handle == theModule->handle) {
		NotifyWarning("AddModuleRef_ModuleMgr: Module already in list.");
		return(TRUE);
	}
	if ((temp = CreateModuleRef_ModuleMgr(theModule)) == NULL) {
		NotifyError("AddModuleRef_ModuleMgr: Could not create temp node.");
		return(FALSE);
	}
	if (p->module->handle > theModule->handle) {
		temp->module = theModule;
		temp->next = p;
		if (p == *theList)	/* Test for head of list. */
			*theList = temp;	
		else
			p = temp;
	} else
		p->next = temp;
	return(TRUE);

}

/**************************** FreeModuleRef ***********************************/

/*
 * This routine removes an ModuleRef from a list.
 */

void
FreeModuleRef_ModuleMgr(ModuleRefPtr *theList, ModuleHandle theHandle)
{
	ModuleRefPtr	p, lastRef;

    if (*theList == NULL)
    	return;
	for (p = *theList; (p != NULL) && (p->module->handle != theHandle); p =
	  p->next)
		lastRef = p;
	if (p->module->handle == theHandle) {
		if (p == *theList) /* Test for head of list. */
			*theList = p->next;
		else
			lastRef->next = p->next;
		free(p);
	}

}

/************************** Free **********************************************/

/*
 * This routine frees the memory, allocated for module structures.
 * It also unregisters the module from the main list.
 */

void
Free_ModuleMgr(ModulePtr *theModule)
{
	if (*theModule == NULL)
		return;
	FreeModuleRef_ModuleMgr(&moduleList, (*theModule)->handle);
	free(*theModule);
	*theModule = NULL;

}

/**************************** FreeAll *****************************************/

/*
 * This routine frees the memory for all of the initialised modules from
 * the main register list.
 * It does not, however, set the pointer variables to NULL - watch this.
 */

void
FreeAll_ModuleMgr(void)
{
	while (moduleList != NULL)
		FreeModuleRef_ModuleMgr(&moduleList, moduleList->module->handle);

}

/************************** RunModel...Null ***********************************/

/*
 * This function connects the input signal to the output signal, without
 * doing any processing.
 */

BOOLEAN
RunModel_ModuleMgr_Null(EarObjectPtr data)
{
	if (data == NULL) {
		NotifyError("RunModel_NULL_ModuleMgr: EarObject not initialised.");
		return(FALSE);
	}	
	SetProcessName_EarObject(data, "Null process module");
	if (!CheckPars_SignalData(data->inSignal)) {
		NotifyError("RunModel_ModuleMgr_Null: Input signal not set correctly.");
		return(FALSE);
	}
	if (data->outSignal != NULL)
		if (!SameType_SignalData_NoDiagnostics(data->inSignal, data->outSignal))
			data->updateCustomersFlag = TRUE;
	data->outSignal = data->inSignal;
    if (data->updateCustomersFlag)
    	UpdateCustomers_EarObject(data);
	return(TRUE);

}

