/***********************
 *
 * ModuleMgr.h
 *
 * This is the header file for the module management module.
 *
 * 28-10-93 L. P. O'Mard
 *
 ***********************/

#ifndef	_MODULEMGR_H
#define _MODULEMGR_H	1
 
/******************************************************************************/
/*************************** Constant Definitions *****************************/
/******************************************************************************/

#define MAX_MODULE_NAME			20	/* Max. characters in module name. */

/******************************************************************************/
/*************************** MacroDefinitions *********************************/
/******************************************************************************/

#define	DoFun(ROUTINE, EAROBJ)			(* EAROBJ->module->ROUTINE)()

#define	DoFun1(ROUTINE, EAROBJ, ARG)	(* EAROBJ->module->ROUTINE)(ARG)

#define	DoFun2(ROUTINE, EAROBJ, ARG1, ARG2)	(* EAROBJ->module->ROUTINE)((ARG1),\
											  (ARG2))

#define	DoFun3(ROUTINE, EAROBJ, ARG1, ARG2, ARG3) \
  (* EAROBJ->module->ROUTINE)((ARG1), (ARG2), (ARG3))

#define	DoProcess(ROUTINE, EAROBJ)		(* EAROBJ->module->ROUTINE)(EAROBJ)

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/

typedef enum {
	
	AMTONE,
	AUDITORYN_SPIKEGEN,
	BASILARM_GAMMAT,
	BASILARM_STONE3,
	BASILARM_STONE4,
	CLICK,
	DATAFILE,
	IHC_MEDDIS86,
	IHC_MEDDIS93,
	IHC_RECPTPOTN,
	NEURON_DENDRITELP,
	NEURON_MCGREGOR,
	NEURON_ARLEKIM,
	NULL_MODULE,
	PREEMPHASIS_BANDPASS,
	PURETONE_BINAURAL,
	PURETONE_MULTI,
	PURETONE_MULTIPULSE,
	PURETONE,
	PURETONE_2,
	STEPFUN,
	NULL_SPECIFIER

} ModuleSpecifier;

typedef unsigned int	ModuleHandle;	/* For module management. */

typedef struct moduleStruct {

	char	name[MAX_MODULE_NAME];
	ModuleHandle	handle;				/* Reference handle for manager. */
	
	/* General, accessible modules. */
	
	BOOLEAN	(* CheckData)(EarObjectPtr data);
	BOOLEAN	(* CheckPars)(void);
	BOOLEAN	(* GenerateSignal)(EarObjectPtr theObject);
	double	(* GetCF)(int channel);
	double	(* GetIndividualFreq)(int index);
	double	(* GetIndividualIntensity)(int index);
	double	(* GetPotentialResponse)(double potential);
	void 	(* PrintPars)(void);
	BOOLEAN	(* ReadPars)(char *fileName);
	BOOLEAN	(* ReadSignal)(char *fileName, EarObjectPtr data);
	BOOLEAN	(* RunModel)(EarObjectPtr data);
	BOOLEAN	(* SetAccomConst)(double theAccomConst);
	BOOLEAN	(* SetActionPotential)(double theActionPotential);
	BOOLEAN	(* SetAmplitude)(double theAmplitude);
	BOOLEAN	(* SetBandwidths)(char *theBandwidthMode, double *theBandwidths,
	  size_t arraySize);
	BOOLEAN	(* SetBaseWideFAtten)(double theBaseWideFAtten);
	BOOLEAN	(* SetBeginSilence)(double theBeginSilence);
	BOOLEAN	(* SetBMTimeConstTm)(double theBMTimeConstTm);
	BOOLEAN	(* SetBRestingCond)(double theBRestingCond);
	BOOLEAN	(* SetBReversalPoten)(double theBReversalPoten);
	BOOLEAN	(* SetCarrierFrequency)(double theCarrierFrequency);
	BOOLEAN	(* SetCellRestingPot)(double theRestingPot);
	BOOLEAN	(* SetCentreFreq)(double *theFrequencies, size_t arraySize);
	BOOLEAN	(* SetCFList)(CFListPtr theCFList);
	BOOLEAN	(* SetClickTime)(double theClickTime);
	BOOLEAN	(* SetCompressionExponent)(double theCompressionExponent);
	BOOLEAN	(* SetCompressionThreshold)(double theCompressionThreshold);
	BOOLEAN	(* SetCutOffFrequency)(double theCutOffFrequency);
	BOOLEAN	(* SetDelayedRectKCond)(double theDelayedRectKCond);
	BOOLEAN	(* SetDuration)(double theDuration);
	BOOLEAN	(* SetEndSilence)(double theEndSilence);
	BOOLEAN	(* SetFiringRate)(double theSetFiringRate);
	BOOLEAN	(* SetFrequencies)(double *theFrequencies);
	BOOLEAN	(* SetFrequency)(double theFrequency);
	BOOLEAN	(* SetIndividualFreq)(int theIndex, double theFrequency);
	BOOLEAN	(* SetIndividualIntensity)(int theIndex, double theIntensity);
	BOOLEAN	(* SetIndividualPhase)(int theIndex, double thePhase);
	BOOLEAN	(* SetIntensities)(double *theIntensities);
	BOOLEAN	(* SetIntensity)(double theIntensity);
	BOOLEAN	(* SetKDecayTConst)(double theKDecayTConst);
	BOOLEAN	(* SetKEquilibriumPot)(double theKEquilibriumPot);
	BOOLEAN	(* SetKRestingCond)(double theKRestingCond);
	BOOLEAN	(* SetKReversalPoten)(double theKReversalPoten);
	BOOLEAN	(* SetLossRate)(double theLossRate);
	BOOLEAN	(* SetLowerCutOffFreq)(double theLowerCutOffFreq);
	BOOLEAN	(* SetMaxFreePool)(double theMaxFreePool);
	BOOLEAN	(* SetMembraneTConst)(double theMembraneTConst);
	BOOLEAN	(* SetModulationFrequency)(double theModulationFrequency);
	BOOLEAN	(* SetNarrowBroadeningCoeff)(double theNarrowBroadeningCoeff);
	BOOLEAN	(* SetNarrowFLPCutOffScale)(double theNarrowFLPCutOffScale);
	BOOLEAN	(* SetNonLinearVConst)(double theNonLinearVConst);
	BOOLEAN	(* SetNumChannels)(int theChannels);
	BOOLEAN	(* SetNumPTones)(int theNumPTones);
	BOOLEAN	(* SetNumPulses)(int theNumPulses);
	BOOLEAN	(* SetOrder)(int theOrder);
	BOOLEAN	(* SetParamQ)(double theParamQ);
	BOOLEAN	(* SetParamV)(double theParamV);
	BOOLEAN	(* SetPercentAM)(double thePercentAM);
	BOOLEAN	(* SetPermConstA)(double thePermConstA);
	BOOLEAN	(* SetPermConstB)(double thePermConstB);
	BOOLEAN	(* SetPermeabilityPH)(double thePermeabilityPH);
	BOOLEAN	(* SetPermeabilityPZ)(double thethePermeabilityPZ);
	BOOLEAN	(* SetPhases)(double *thePhases);
	BOOLEAN	(* SetPhaseDifference)(double thePhaseDifference);
	BOOLEAN	(* SetPulseDuration)(double thePulseDuration);
	BOOLEAN	(* SetRanSeed)(long theRanSeed);
	BOOLEAN	(* SetRecoveryRate)(double theRecoveryRate);
	BOOLEAN	(* SetRefractoryPeriod)(double theRefractoryPeriod);
	BOOLEAN	(* SetReleaseRate)(double theReleaseRate);
	BOOLEAN	(* SetReplenishRate)(double theReplenishRate);
	BOOLEAN	(* SetRepetitionPeriod)(double theRepetitionPeriod);
	BOOLEAN	(* SetReprocessRate)(double theReprocessRate);
	BOOLEAN	(* SetRestingThreshold)(double theRestingThreshold);
	BOOLEAN	(* SetSamplingInterval)(double theSamplingInterval);
	BOOLEAN	(* SetSignalMultiplier)(double theUpperCutOffFreq);
	BOOLEAN	(* SetThresholdTConst)(double theThresholdTConst);
	BOOLEAN	(* SetUpperCutOffFreq)(double theUpperCutOffFreq);
	BOOLEAN	(* SetWideAttenuation)(double theWideAttenuation);
	BOOLEAN	(* SetWideBroadeningCoeff)(double theWideBroadeningCoeff);
	BOOLEAN	(* SetWideFAttenScale)(double theWideFAttenScale);
	BOOLEAN	(* SetWideFCF)(double theWideFCF);
	BOOLEAN	(* SetWideFCentreFreq)(double theWideFCentreFreq);
	BOOLEAN	(* SetWideFWidth)(double theWideFWidth);
	BOOLEAN	(* SetWideFLPCutOffScale)(double theWideFLPCutOffScale);
	BOOLEAN	(* SetWideFRefCF)(double theWideFRefCF);
	BOOLEAN	(* SetWideHighFCutOff)(double theWideHighFCutOff);
	BOOLEAN	(* SetWideLowFCutOff)(double theWideLowFCutOff);
	BOOLEAN	(* WriteOutSignal)(char *fileName, EarObjectPtr data);

} Module;

#ifndef MODULE_PTR
#	define MODULE_PTR

	typedef struct moduleStruct  *ModulePtr;/* Pre-referencing of module.  */
	
#endif

typedef struct moduleNode {

	ModulePtr			module;
	struct moduleNode	*next;
	
} ModuleRef, *ModuleRefPtr;

/******************************************************************************/
/*************************** External Variables *******************************/
/******************************************************************************/

/******************************************************************************/
/*************************** Function Prototypes ******************************/
/******************************************************************************/

BOOLEAN		AddModuleRef_ModuleMgr(ModuleRefPtr *theList, ModulePtr theModule);

ModuleRefPtr	CreateModuleRef_ModuleMgr(ModulePtr theModule);

void		Free_ModuleMgr(ModulePtr *theModule);

void		FreeAll_ModuleMgr(void);

void		FreeModuleRef_ModuleMgr(ModuleRefPtr *theList, ModuleHandle
			  theHandle);

ModuleSpecifier	IdentifyModuleName_ModuleMgr(char *name);

ModulePtr	Init_ModuleMgr(char *name);

void *		NoFunction_ModuleMgr(void);

void *		NullFunction_ModuleMgr(void);

BOOLEAN		RunModel_ModuleMgr_Null(EarObjectPtr data);

void		SetAMTone_ModuleMgr(ModulePtr theModule);

void		SetAuditoryN_SpikeGen_ModuleMgr(ModulePtr theModule);

void		SetBasilarM_GammaT_ModuleMgr(ModulePtr theModule);

void		SetBasilarM_Stone3_ModuleMgr(ModulePtr theModule);

void		SetBasilarM_Stone4_ModuleMgr(ModulePtr theModule);

void		SetBPureTone_ModuleMgr(ModulePtr theModule);

void		SetClick_ModuleMgr(ModulePtr theModule);

void		SetDataFile_ModuleMgr(ModulePtr theModule);

void		SetDefault_ModuleMgr(ModulePtr module, void *(* DefaultFunc)(void));

void		SetIHC_Meddis86_ModuleMgr(ModulePtr theModule);

void		SetIHC_Meddis93_ModuleMgr(ModulePtr theModule);

void		SetIHC_RecptPotn_ModuleMgr(ModulePtr theModule);

void		SetMPureTone_ModuleMgr(ModulePtr theModule);

void		SetNeuron_DendriteLP_ModuleMgr(ModulePtr theModule);

void		SetNeuron_McGregor_ModuleMgr(ModulePtr theModule);

void		SetNeuron_ArleKim_ModuleMgr(ModulePtr theModule);

void		SetNull_ModuleMgr(ModulePtr module);

void		SetMPPureTone_ModuleMgr(ModulePtr theModule);

void		SetPreEmphasis_BandPass_ModuleMgr(ModulePtr theModule);

void		SetPureTone_ModuleMgr(ModulePtr theModule);

void		SetPureTone2_ModuleMgr(ModulePtr theModule);

void		SetStepFun_ModuleMgr(ModulePtr theModule);

#endif
