/*******
 *
 * Mo2HairCell.h
 *
 * This header file contains the Meddis 86 hair cell module.
 * The parameters A and B have been changed so that the hair cell responds
 * correctly for  Intensity levels in dB (SPL).
 *
 * by Lowel O'Mard 18-2-93.
 *
 ********/

#ifndef	_MO2HAIRCELL_H
#define _MO2HAIRCELL_H	1
 
/******************************************************************************/
/*************************** Constant Definitions *****************************/
/******************************************************************************/

#define	MEDDIS93_MAX_DT	0.00005		/* Magic number for model */

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/

typedef struct {

	BOOLEAN	numberOfRunsFlag, replenishRateFlag;
	BOOLEAN	lossRateFlag, reprocessRateFlag, recoveryRateFlag, maxFreePoolFlag;
	BOOLEAN	permeabilityPHFlag, permeabilityPZFlag, refractoryPeriodFlag;

	int		numberOfRuns;		/* No. of repeated runs for model. */
	int		maxFreePool_M;		/* Max. no. of transm. packets in free pool */
	double	replenishRate_y;	/* Replenishment rate (units per second). */
	double	lossRate_l;			/* Loss rate (units per second). */
	double	reprocessRate_x;	/* Reprocessing rate (units per second). */
	double	recoveryRate_r;		/* Recovery rate (units per second). */
	double	permeabilityPH_h;	/* Permeability constant. */
	double	permeabilityPZ_z;	/* Permeability constant. */
	double	refractoryPeriod;	/*  Non-ejection period. */

} HairCell2;

typedef struct {

		int	cleftC;			/* Quantity of transmitter in the cleft */
		int	reservoirQ;		/* Quantity of transmitter in the free pool */
		int	reprocessedW;	/* Quantity of transm. in the reproc. store */
		double	refractoryTimer;	/* Keeps track of non-firing period. */

} HairCellVars2;
	
/******************************************************************************/
/*************************** External Variables *******************************/
/******************************************************************************/

extern	HairCell2		hairCell2;

/******************************************************************************/
/*************************** Function Prototypes ******************************/
/******************************************************************************/

BOOLEAN	CheckData_IHC_Meddis93(EarObjectPtr data);

BOOLEAN	CheckPars_IHC_Meddis93(void);

void	InitSpontOut_IHC_Meddis93(EarObjectPtr data,
		  HairCellVars2 hCChannels[]);

void	PrintPars_IHC_Meddis93(void);

BOOLEAN	ReadPars_IHC_Meddis93(char *fileName);

BOOLEAN	RunModel_IHC_Meddis93(EarObjectPtr data);

BOOLEAN	SetLossRate_IHC_Meddis93(double theLossRate);

BOOLEAN	SetMaxFreePool_IHC_Meddis93(double theMaxFreePool);

BOOLEAN	SetNumberOfRuns_IHC_Meddis93(int theNumberOfRuns);

BOOLEAN	SetPars_IHC_Meddis93(int theNumberOfRuns, double y, double l,
		  double x, double r, double M, double h, double z,
		  double refractoryPeriod);
		  
BOOLEAN	SetPermeabilityPH_IHC_Meddis93(double thePermeabilityPH);

BOOLEAN	SetPermeabilityPZ_IHC_Meddis93(double thePermeabilityPZ);

BOOLEAN	SetRefractoryPeriod_IHC_Meddis93(double theRefractoryPeriod);

BOOLEAN	SetReplenishRate_IHC_Meddis93(double theReplenishRate);

BOOLEAN	SetReprocessRate_IHC_Meddis93(double theReprocessRate);

BOOLEAN	SetRecoveryRate_IHC_Meddis93(double theRecoveryRate);

#endif
