/*******
 *
 * Mo3BasilarM.h
 *
 * This header file contains the definitions for the Basilar membrane module.
 * It uses a Stone Composite filter to model the basilar membrane behaviour.
 *
 * by Lowel O'Mard 11-3-93.
 *
 ********/

#ifndef	_MO3BASILARM_H
#define _MO3BASILARM_H	1

/******************************************************************************/
/*************************** Constant definitions *****************************/
/******************************************************************************/

#define	BM3_BASE_ATTEN			-67.0	/* Attenuation at reference CF (dB). */
#define	ATTENUATION_SCALE		12.25	/* Scaled increase in attenuation. */
#define REFERENCE_CF			8000.0	/* Reference/base CF (Hz). */

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/ 

typedef struct {

	BOOLEAN	orderFlag;
	BOOLEAN	narrowBroadeningCoeffFlag, wideBroadeningCoeffFlag;
	BOOLEAN	compressionThresholdFlag, compressionExponentFlag;
	BOOLEAN	wideFCFFlag, baseWideFAttenFlag, wideFAttenScaleFlag;
	BOOLEAN	wideFRefCFFlag, narrowFLPCutOffScaleFlag, wideFLPCutOffScaleFlag;
	BOOLEAN	theCFsFlag;
	int		order;					/* Total (narrow filter) line order. */
	double	narrowBroadeningCoeff;	/* Narrow broadening coefficient */
	double	wideBroadeningCoeff;	/* Wide broadening coefficient */
	double	compressionThreshold;	/* Point after which compression occurs. */
	double	compressionExponent;	/* Defines ratio of compression ratio. */
	double	wideFCF;				/* CF of the Wide filter. */
	double	wideFRefCF;				/* Reference/base CF for wide filter (Hz).*/
	double	baseWideFAtten;			/* Base wide filter attenuation (dB SPL). */
	double	wideFAttenScale;		/* Scaling for wide filter attenuation. */
	double	narrowFLPCutOffScale;	/* - filter low-pass cut-off scale. */
	double	wideFLPCutOffScale;		/* - filter low-pass cut-off scale. */
	CFListPtr		theCFs;			/* Pointer to centre frequency structure. */
	EarObjectPtr	wideF;			/* Extra signal for wide filter. */

} BasilarM3;

/******************************************************************************/
/*************************** External Variables *******************************/
/******************************************************************************/ 
extern	BasilarM3	basilarM3;

/******************************************************************************/
/*************************** Function Prototypes ******************************/
/******************************************************************************/ 

BOOLEAN	CheckPars_BasilarM_Stone3(void);

void	FreeCoeffs_BasilarM_Stone3(GammaToneCoeffs *narrowGammaTone[],
		  GammaToneCoeffs *wideGammaTone[], TwoPoleCoeffs *butterworthLP[],
		  TwoPoleCoeffs *besselLP[], EarObjectPtr data);

double	GetCF_BasilarM_Stone3(int channel);

void	InitCoeffs_BasilarM_Stone3(GammaToneCoeffs *narrowGammaTone[],
		  GammaToneCoeffs *wideGammaTone[], TwoPoleCoeffs *butterworthLP[],
		  TwoPoleCoeffs *besselLP[], EarObjectPtr data);
		  
void PrintPars_BasilarM_Stone3(void);

BOOLEAN	ReadPars_BasilarM_Stone3(char *fileName);

BOOLEAN	RunModel_BasilarM_Stone3(EarObjectPtr data);

BOOLEAN	SetBaseWideFAtten_BasilarM_Stone3(double theBaseWideFAtten);

BOOLEAN	SetCentreFreq_BasilarM_Stone3(double *theFrequencies,
		  size_t arraySize);

BOOLEAN SetCFList_BasilarM_Stone3(CFList *theCFList);

BOOLEAN	SetCompressionExponent_BasilarM_Stone3(double theCompressionExponent);

BOOLEAN	SetCompressionThreshold_BasilarM_Stone3(double theCompressionThreshold);

BOOLEAN	SetNumChannels_BasilarM_Stone3(int theChannels);

BOOLEAN	SetNarrowBroadeningCoeff_BasilarM_Stone3(double 
		  theNarrowBroadeningCoeff);

BOOLEAN	SetNarrowFLPCutOffScale_BasilarM_Stone3(double theNarrowFLPCutOffScale);

BOOLEAN	SetOrder_BasilarM_Stone3(int theOrder);

BOOLEAN	SetPars_BasilarM_Stone3(int theOrder, double theNarrowBroadeningCoeff,
		  double theWideBroadeningCoeff, double theCompressionThreshold,
		  double theCompressionExponent, double wideFCF, double wideFRefCF,
		  double baseWideFAtten, double wideFAttenScale,
		  double narrowFLPCutOffScale, double wideFLPCutOffScale,
		  int theChannels, ...);
		  
BOOLEAN	SetWideBroadeningCoeff_BasilarM_Stone3(double theWideBroadeningCoeff);

BOOLEAN	SetWideFAttenScale_BasilarM_Stone3(double theWideFAttenScale);

BOOLEAN	SetWideFCF_BasilarM_Stone3(double theWideFCF);

BOOLEAN	SetWideFLPCutOffScale_BasilarM_Stone3(double theWideFLPCutOffScale);

BOOLEAN	SetWideFRefCF_BasilarM_Stone3(double theWideFRefCF);

#endif
