/*******
 *
 * Mo4BasilarM.h
 *
 * This header file contains the definitions for the Basilar membrane module.
 * It uses a Stone Composite filter to model the basilar membrane behaviour.
 *
 * by Lowel O'Mard 11-3-93.
 *
 ********/

#ifndef	_MO4BASILARM_H
#define _MO4BASILARM_H	1

/******************************************************************************/
/*************************** Constant definitions *****************************/
/******************************************************************************/

#define	BM4_ATTENUATION_K1	-40.798		/* For wide filter attenuation */
#define BM4_ATTENUATION_K2	-6.311e-3	/*  - as above. */
#define BM4_ATTENUATION_K3	2.068e-7	/*  - as above. */

#define BM4_BROADENING_K1	0.957		/* For wide filter braodening. */
#define BM4_BROADENING_K2	4.111e-4	/* -as above. */
#define BM4_BROADENING_K3	-1.959e-8	/* -as above. */

#define BM4_NARROW_BROADENING	2.0		/* Constant wide filter broadening. */

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/ 

typedef struct {

	BOOLEAN	orderFlag;
	BOOLEAN	compressionThresholdFlag, compressionExponentFlag;
	BOOLEAN	wideFCFFlag, narrowFLPCutOffScaleFlag, wideFLPCutOffScaleFlag;
	BOOLEAN	theCFsFlag;
	int		order;					/* Total (narrow filter) line order. */
	double	compressionThreshold;	/* Point after which compression occurs. */
	double	compressionExponent;	/* Defines ratio of compression ratio. */
	double	wideFCF;				/* CF of the Wide filter. */
	double	narrowFLPCutOffScale;	/* - filter low-pass cut-off scale. */
	double	wideFLPCutOffScale;		/* - filter low-pass cut-off scale. */
	CFListPtr	theCFs;				/* Pointer to centre frequency structure. */
	EarObjectPtr	wideF;			/* Extra signal for wide filter. */

} BasilarM4;

/******************************************************************************/
/*************************** External Variables *******************************/
/******************************************************************************/

extern	BasilarM4	basilarM4;

/******************************************************************************/
/*************************** Function Prototypes ******************************/
/******************************************************************************/

BOOLEAN	CheckPars_BasilarM_Stone4(void);

void	FreeCoeffs_BasilarM_Stone4(GammaToneCoeffs *narrowGammaTone[],
		  GammaToneCoeffs *wideGammaTone[], TwoPoleCoeffs *butterworthLP[],
		  TwoPoleCoeffs *besselLP[],EarObjectPtr data);

double	GetCF_BasilarM_Stone4(int channel);

void	InitCoeffs_BasilarM_Stone4(GammaToneCoeffs *narrowGammaTone[],
		  GammaToneCoeffs *wideGammaTone[], TwoPoleCoeffs *butterworthLP[],
		  TwoPoleCoeffs *besselLP[], EarObjectPtr data);
		  
void	PrintPars_BasilarM_Stone4(void);

BOOLEAN	ReadPars_BasilarM_Stone4(char *fileName);

BOOLEAN	RunModel_BasilarM_Stone4(EarObjectPtr data);

BOOLEAN	SetCentreFreq_BasilarM_Stone4(double *theFrequencies,
		  size_t arraySize);

BOOLEAN SetCFList_BasilarM_Stone4(CFList *theCFList);

BOOLEAN	SetCompressionExponent_BasilarM_Stone4(double theCompressionExponent);

BOOLEAN	SetCompressionThreshold_BasilarM_Stone4(double theCompressionThreshold);

BOOLEAN	SetNumChannels_BasilarM_Stone4(int theChannels);

BOOLEAN	SetNarrowFLPCutOffScale_BasilarM_Stone4(double theNarrowFLPCutOffScale);

BOOLEAN	SetOrder_BasilarM_Stone4(int theOrder);

BOOLEAN	SetPars_BasilarM_Stone4(int theOrder, double theCompressionThreshold,
		  double theCompressionExponent, double wideFCF,
		  double narrowFLPCutOffScale, double wideFLPCutOffScale,
		  int theChannels, ...);
		  
BOOLEAN	SetWideFCF_BasilarM_Stone4(double theWideFCF);

BOOLEAN	SetWideFLPCutOffScale_BasilarM_Stone4(double theWideFLPCutOffScale);

#endif
