/*******
 *
 * MoBasilarM.h
 *
 * This header file contains the definitions for the Basilar membrane module.
 * It uses a gamma tone filter to model the basilar membrane behaviour.
 *
 * by Lowel O'Mard 11-3-93.
 *
 ********/

#ifndef	_MOBASILARM_H
#define _MOBASILARM_H	1

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/

typedef struct {

	BOOLEAN		orderFlag, theCFsFlag;
	int			order;				/* Defines the Q value of the gamma tone
									 * filters. */
	CFListPtr	theCFs;			/* Pointer to centre frequency structure. */

	/* Functions & Routines */

	BOOLEAN	(* CheckPars)(void);
	double	(* GetCF)(int channel);
	void	(* PrintPars)(void);
	BOOLEAN	(* ReadPars)(char *fileName);
	BOOLEAN	(* RunModel)(EarObjectPtr data);
	BOOLEAN	(* SetCentreFreq)(double *theFrequencies, size_t arraySize);
	BOOLEAN (* SetCFList)(CFList *theCFList);
	BOOLEAN	(* SetNumChannels)(int theChannels);
	BOOLEAN	(* SetOrder)(int theOrder);
	BOOLEAN	(* SetPars)(int theOrder, int theChannels, ...);

} BasilarM;

/******************************************************************************/
/*************************** External Variables *******************************/
/******************************************************************************/

extern	BasilarM	basilarM;

/******************************************************************************/
/*************************** Function Prototypes ******************************/
/******************************************************************************/

BOOLEAN	CheckPars_BasilarM_GammaT(void);

double	GetCF_BasilarM_GammaT(int channel);

void	PrintPars_BasilarM_GammaT(void);

BOOLEAN	ReadPars_BasilarM_GammaT(char *fileName);

BOOLEAN	RunModel_BasilarM_GammaT(EarObjectPtr data);

BOOLEAN	SetCentreFreq_BasilarM_GammaT(double *theFrequencies,
			  size_t arraySize);

BOOLEAN SetCFList_BasilarM_GammaT(CFList *theCFList);

BOOLEAN	SetNumChannels_BasilarM_GammaT(int theChannels);

BOOLEAN	SetOrder_BasilarM_GammaT(int theOrder);

BOOLEAN	SetPars_BasilarM_GammaT(int theOrder, int theChannels, ...);

#endif
