/*******
 *
 * MoHairCell.c
 *
 * This module contains the routines for the Meddis 86 Hair cell model module.
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	12th July 1993.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 ********/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "LUTEar.h"
#include "SignalData.h"
#include "EarObject.h"
#include "MoHairCell.h"
#include "FiParFile.h"

/******************************************************************************/
/*************************** Global Variables *********************************/
/******************************************************************************/

HairCell		hairCell = {

					FALSE,	/* permConstAFlag */
					FALSE,	/* permConstBFlag */
					FALSE,	/* releaseRateFlag */
					FALSE,	/* replenishRateFlag */
					FALSE,	/* lossRateFlag */
					FALSE,	/* reprocessRateFlag */
					FALSE,	/* recoveryRateFlag */
					FALSE,	/* maxFreePoolFlag */
					FALSE,	/* firingRateFlag */
					
					0.0,	/* permConst_A */
					0.0,	/* permConst_B */
					0.0,	/* releaseRate_g  */ 
					0.0,	/* replenishRate_y  */ 
					0.0,	/* lossRate_l */
					0.0,	/* reprocessRate_x */
					0.0,	/* recoveryRate_r */
					0.0,	/* maxFreePool_M */
					0.0		/* firingRate_h */

				};

/******************************************************************************/
/*************************** Subroutines and Functions ************************/
/******************************************************************************/

/********************************* SetPermConstA ******************************/

/*
 * This function sets the module's permeability constant A parameter.
 * It returns TRUE if the operation is successful.
 */

BOOLEAN
SetPermConstA_IHC_Meddis86(double thePermConstA)
{
	hairCell.permConstAFlag = TRUE;
	hairCell.permConst_A = thePermConstA;
	return(TRUE);

}

/********************************* SetPermConstB ******************************/

/*
 * This function sets the module's permeability constant B parameter.
 * It returns TRUE if the operation is successful.
 */

BOOLEAN
SetPermConstB_IHC_Meddis86(double thePermConstB)
{
	hairCell.permConstBFlag = TRUE;
	hairCell.permConst_B = thePermConstB;
	return(TRUE);

}

/********************************* SetReleaseRate *****************************/

/*
 * This function sets the module's release rate constant, r, parameter.
 * It returns TRUE if the operation is successful.
 */

BOOLEAN
SetReleaseRate_IHC_Meddis86(double theReleaseRate)
{
	hairCell.releaseRateFlag = TRUE;
	hairCell.releaseRate_g = theReleaseRate;
	return(TRUE);

}

/********************************* SetReplenishRate ***************************/

/*
 * This function sets the module's replenish rate, y, parameter.
 * It returns TRUE if the operation is successful.
 */

BOOLEAN
SetReplenishRate_IHC_Meddis86(double theReplenishRate)
{
	hairCell.replenishRateFlag = TRUE;
	hairCell.replenishRate_y = theReplenishRate;
	return(TRUE);

}

/********************************* SetLossRate ********************************/

/*
 * This function sets the module's loss rate, l, parameter.
 * It returns TRUE if the operation is successful.
 */

BOOLEAN
SetLossRate_IHC_Meddis86(double theLossRate)
{
	hairCell.lossRateFlag = TRUE;
	hairCell.lossRate_l = theLossRate;
	return(TRUE);

}

/********************************* SetReprocessRate ***************************/

/*
 * This function sets the module's permeability constant B parameter.
 * It returns TRUE if the operation is successful.
 */

BOOLEAN
SetReprocessRate_IHC_Meddis86(double theReprocessRate)
{
	hairCell.reprocessRateFlag = TRUE;
	hairCell.reprocessRate_x = theReprocessRate;
	return(TRUE);

}

/********************************* SetRecoveryRate ****************************/

/*
 * This function sets the module's permeability constant B parameter.
 * It returns TRUE if the operation is successful.
 */

BOOLEAN
SetRecoveryRate_IHC_Meddis86(double theRecoveryRate)
{
	hairCell.recoveryRateFlag = TRUE;
	hairCell.recoveryRate_r = theRecoveryRate;
	return(TRUE);

}

/********************************* SetMaxFreePool *****************************/

/*
 * This function sets the module's permeability constant B parameter.
 * It returns TRUE if the operation is successful.
 */

BOOLEAN
SetMaxFreePool_IHC_Meddis86(double theMaxFreePool)
{
	hairCell.maxFreePoolFlag = TRUE;
	hairCell.maxFreePool_M = theMaxFreePool;
	return(TRUE);

}

/********************************* SetFiringRate ******************************/

/*
 * This function sets the module's permeability constant B parameter.
 * It returns TRUE if the operation is successful.
 */

BOOLEAN
SetFiringRate_IHC_Meddis86(double theFiringRate)
{
	hairCell.firingRateFlag = TRUE;
	hairCell.firingRate_h = theFiringRate;
	return(TRUE);

}

/********************************* SetPars ************************************/

/*
 * This function sets all the module's parameters.
 */

BOOLEAN
SetPars_IHC_Meddis86(double A, double B, double g, double y, double l, double x,
  double r, double M, double h)
{
	BOOLEAN	ok;
	
	ok = TRUE;
	if (!SetPermConstA_IHC_Meddis86(A))
		ok = FALSE;
	if (!SetPermConstB_IHC_Meddis86(B))
		ok = FALSE;
	if (!SetReleaseRate_IHC_Meddis86(g))
		ok = FALSE;
	if (!SetReplenishRate_IHC_Meddis86(y))
	    ok = FALSE;
	if (!SetLossRate_IHC_Meddis86(l))
		ok = FALSE;
	if (!SetReprocessRate_IHC_Meddis86(x))
		ok = FALSE;
	if (!SetRecoveryRate_IHC_Meddis86(r))
		ok = FALSE;
	if (!SetMaxFreePool_IHC_Meddis86(M))
		ok = FALSE;
	if (!SetFiringRate_IHC_Meddis86(h))
		ok = FALSE;
	return(ok);
	  
}

/********************************* CheckPars **********************************/

/*
 * This routine checks that all of the parameters for the module are set.
 * Because this module is defined by its coeffients, default values are always
 * set if none of the parameters are set, otherwise all of them must be
 * individually set.
 */

BOOLEAN
CheckPars_IHC_Meddis86(void)
{
	BOOLEAN	ok;
	
	ok = TRUE;
	if (!hairCell.permConstAFlag) {
		NotifyError("CheckPars_IHC_Meddis86: Permeability constant, A, "\
		  "not correctly set.");
		ok = FALSE;
	}
	if (!hairCell.permConstBFlag) {
		NotifyError("CheckPars_IHC_Meddis86: Permeability constant, B, "\
		  "not correctly set.");
		ok = FALSE;
	}
	if (!hairCell.releaseRateFlag) {
		NotifyError("CheckPars_IHC_Meddis86: Release rate constant, g, "\
		  "not correctly set.");
		ok = FALSE;
	}
	if (!hairCell.replenishRateFlag) {
		NotifyError("CheckPars_IHC_Meddis86: Replenish rate constant, y, "\
		  "not correctly set.");
		ok = FALSE;
	}
	if (!hairCell.lossRateFlag) {
		NotifyError("CheckPars_IHC_Meddis86: Loss rate constant, l, "\
		  "not correctly set.");
		ok = FALSE;
	}
	if (!hairCell.reprocessRateFlag) {
		NotifyError("CheckPars_IHC_Meddis86: Reprocessing rate constant, x, "\
		  "not correctly set.");
		ok = FALSE;
	}
	if (!hairCell.recoveryRateFlag) {
		NotifyError("CheckPars_IHC_Meddis86: Recovery rate constant, r, "\
		  "not correctly set.");
		ok = FALSE;
	}
	if (!hairCell.maxFreePoolFlag) {
		NotifyError("CheckPars_IHC_Meddis86: Max. free transmitter in pool "\
		  "constant, M, not correctly set.");
		ok = FALSE;
	}
	if (!hairCell.firingRateFlag) {
		NotifyError("CheckPars_IHC_Meddis86: Firing rate constant, h, "\
		  "not correctly set.");
		ok = FALSE;
	}
	return(ok);
		
}

/****************************** PrintPars *************************************/

/*
 * This program prints the parameters of the module to the standard output.
 */
 
void
PrintPars_IHC_Meddis86(void)
{
	if (!CheckPars_IHC_Meddis86()) {
		NotifyError("PrintPars_IHC_Meddis86: Parameters have not been "\
		  "correctly set.");
		return;
	}
	printf("Meddis 86 Inner Hair Cell Module Parameters:-\n");
	printf("\tPermeability constant, A = %g,\tand B = %g\n",
	  hairCell.permConst_A, hairCell.permConst_B);
	printf("\tRelease rate, g = %g /s,\tReplenishment rate, y = %g /s\n",
	  hairCell.releaseRate_g, hairCell.replenishRate_y);
	printf("\tLoss rate, l = %g /s,\tReprocessing rate, x = %g /s\n",
	  hairCell.lossRate_l, hairCell.reprocessRate_x);
	printf("\tRecovery rate, r = %g /s,\t Firing rate, h = %g spikes/s\n",
	  hairCell.recoveryRate_r, hairCell.firingRate_h);
	printf("\tMaximum transmitter packets in free pool, M = %g.\n",
	  hairCell.maxFreePool_M);

}

/****************************** ReadPars **************************************/

/*
 * This program reads a specified number of parameters from a file.
 * It returns FALSE if it fails in any way.
 */
 
BOOLEAN
ReadPars_IHC_Meddis86(char *fileName)
{
	BOOLEAN	ok;
	double	permConst_A, permConst_B, releaseRate_g, replenishRate_y;
	double	lossRate_l, reprocessRate_x, recoveryRate_r, maxFreePool_M;
	double	firingRate_h;
    FILE    *fp;
    
    if ((fp = fopen(fileName, "r")) == NULL) {
        NotifyError("ReadPars_IHC_Meddis86: Cannot open data file '%s'.\n",
		  fileName);
		return(FALSE);
    }
    NotifyWarning("ReadPars_IHC_Meddis86: Reading from '%s'", fileName);
    Init_ParFile();
	ok = TRUE;
	if (!GetPars_ParFile(fp, "%lf", &permConst_A))
		ok = FALSE;
	if (!GetPars_ParFile(fp, "%lf", &permConst_B))
		ok = FALSE;
	if (!GetPars_ParFile(fp, "%lf", &releaseRate_g))
		ok = FALSE;
	if (!GetPars_ParFile(fp, "%lf", &replenishRate_y))
		ok = FALSE;
	if (!GetPars_ParFile(fp, "%lf", &lossRate_l))
		ok = FALSE;
	if (!GetPars_ParFile(fp, "%lf", &reprocessRate_x))
		ok = FALSE;
	if (!GetPars_ParFile(fp, "%lf", &recoveryRate_r))
		ok = FALSE;
	if (!GetPars_ParFile(fp, "%lf", &maxFreePool_M))
		ok = FALSE;
	if (!GetPars_ParFile(fp, "%lf", &firingRate_h))
		ok = FALSE;
    fclose(fp);
    Free_ParFile();
	if (!ok) {
		NotifyError("ReadPars_IHC_Meddis86: Not enough lines, or invalid "\
		  "parameters, in module parameter file '%s'.", fileName);
		return(FALSE);
	}
	if (!SetPars_IHC_Meddis86(permConst_A, permConst_B, releaseRate_g,
	  replenishRate_y, lossRate_l, reprocessRate_x, recoveryRate_r,
	  maxFreePool_M, firingRate_h)) {
		NotifyError("ReadPars_IHC_Meddis86: Could not set parameters.");
		return(FALSE);
	}
	return(TRUE);
    
}

/********************************* CheckData **********************************/

/*
 * This routine checks that the input signal is correctly initialised
 * (using CheckRamp_SignalData), and determines whether the parameter values
 * are valid for the signal sampling interval.
 */

BOOLEAN
CheckData_IHC_Meddis86(EarObjectPtr data)
{
	double	dt;
	
	if (data == NULL) {
		NotifyError("CheckData_IHC_Meddis86: EarObject not initialised.");
		return(FALSE);
	}	
	if (!CheckRamp_SignalData(data->inSignal))
		return(FALSE);
	dt = data->inSignal->dt;
	if (dt > MEDDIS86_MAX_DT) {
		NotifyError("CheckData_IHC_Meddis86: Maximum sampling interval "\
		  "exceeded.");
		return(FALSE);
	}
	return(TRUE);
	
}

/********************************* InitSpontOut *******************************/

/*
 * This routine initialises the hair cells to the spontaneous firing rate.
 * The Spontaneous firing rate is determined from the equilbrium state of the
 * system with no input: the reservoir contents do not change in time.
 */

void
InitSpontOut_IHC_Meddis86(EarObjectPtr data, HairCellVars hCChannels[])
{
	ChanLen	i;
	double	spontPerm_k0, spontCleft_c0, spontFreePool_q0, spontReprocess_w0;
	HairCell	*hC;
	
	hC = &hairCell;		/* Shorter variable for long formulae. */

	spontPerm_k0 = hC->releaseRate_g * hC->permConst_A / (hC->permConst_A +
	  hC->permConst_B);
	spontCleft_c0 = (hC->maxFreePool_M * hC->replenishRate_y * spontPerm_k0 /
	  (hC->replenishRate_y * (hC->lossRate_l + hC->recoveryRate_r) +
	  spontPerm_k0 * hC->lossRate_l));
	spontFreePool_q0 = (spontCleft_c0 * (hC->lossRate_l + hC->recoveryRate_r) /
	  spontPerm_k0);
	spontReprocess_w0 = (spontCleft_c0 * hC->recoveryRate_r /
	  hC->reprocessRate_x);
	
	for (i = 0; i < data->outSignal->numChannels; i++) {
		hCChannels[i].cleftC = spontCleft_c0;
		hCChannels[i].reservoirQ = spontFreePool_q0;
		hCChannels[i].reprocessedW = spontReprocess_w0;
	}
	
}

/********************************* RunModel ***********************************/

/*
 * This routine runs the input signal through the model and puts the result
 * into the output signal.  It checks that all initialisation has been
 * correctly carried out by calling the appropriate checking routines.
 */

BOOLEAN
RunModel_IHC_Meddis86(EarObjectPtr data)
{
	int		i, j;
	double	dt, ymdt, xdt, ydt, l_Plus_rdt, rdt, kdt, st_Plus_A, gdt, hdt;
	register	double	replenish, reprocessed, ejected, reUptake;
	register	double	reUptakeAndLost;
	ChanData	*inPtr, *outPtr;
	HairCellVars	hCChannels[MAX_CHANNELS];
	
	if (!CheckPars_IHC_Meddis86())		
		return(FALSE);
	if (!CheckData_IHC_Meddis86(data)) {
		NotifyError("RunModel_IHC_Meddis86: Process data invalid.");
		return(FALSE);
	}
	if (!InitOutSignal_EarObject(data, data->inSignal->numChannels,
	  data->inSignal->length, data->inSignal->dt)) {
		NotifyError("RunModel_IHC_Meddis86: Could not initialise output "\
		  "signal.");
		return(FALSE);
	}
	SetProcessName_EarObject(data, "Meddis 86 hair cell");
	dt = data->outSignal->dt;
	
	ymdt = hairCell.replenishRate_y * hairCell.maxFreePool_M * dt;
	xdt = hairCell.reprocessRate_x * dt;
	ydt = hairCell.replenishRate_y * dt;
	l_Plus_rdt = (hairCell.lossRate_l + hairCell.recoveryRate_r) * dt;
	rdt = hairCell.recoveryRate_r * dt;
	gdt = hairCell.releaseRate_g * dt;
	hdt = hairCell.firingRate_h * dt;
	
	InitSpontOut_IHC_Meddis86(data, hCChannels);
	for (i = 0; i < data->outSignal->numChannels; i++) {
		inPtr = data->inSignal->channel[i];
		outPtr = data->outSignal->channel[i];
		for (j = 0; j < data->outSignal->length; j++) {
			if ((st_Plus_A = *inPtr++ + hairCell.permConst_A) > 0)
				kdt = gdt * st_Plus_A / (st_Plus_A + hairCell.permConst_B);
			else
				kdt = 0.0;
				
			if (hCChannels[i].reservoirQ < hairCell.maxFreePool_M)
				replenish = ymdt - ydt * hCChannels[i].reservoirQ;
			else
				replenish = 0.0;
			reprocessed = xdt * hCChannels[i].reprocessedW;
			ejected = kdt * hCChannels[i].reservoirQ;
			reUptake = rdt * hCChannels[i].cleftC;
			reUptakeAndLost = l_Plus_rdt * hCChannels[i].cleftC;

			hCChannels[i].reservoirQ += replenish - ejected +
			  reprocessed;
			hCChannels[i].cleftC += ejected - reUptakeAndLost;
			hCChannels[i].reprocessedW += reUptake - reprocessed;
			
			/* Spike prob. */
			*outPtr++ = (ChanData) (hdt * hCChannels[i].cleftC);
		}
	}
    if (data->updateCustomersFlag)
    	UpdateCustomers_EarObject(data);
	return(TRUE);
		
} /* RunModel_iHC_Meddis86 */
