/*******
 *
 * MoHairCell.h
 *
 * This header file contains the Meddis 86 hair cell module.
 * The parameters A and B have been changed so that the hair cell responds
 * correctly for  Intensity levels in dB (SPL).
 *
 * by Lowel O'Mard 18-2-93.
 *
 ********/

#ifndef	_MOHAIRCELL_H
#define _MOHAIRCELL_H	1
 
/******************************************************************************/
/*************************** Constant Definitions *****************************/
/******************************************************************************/

#define	MEDDIS86_MAX_DT	0.0001		/* Magic number for model */

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/

typedef struct {

	BOOLEAN	permConstAFlag, permConstBFlag, releaseRateFlag, replenishRateFlag;
	BOOLEAN	lossRateFlag, reprocessRateFlag, recoveryRateFlag, maxFreePoolFlag;
	BOOLEAN	firingRateFlag;
	
	double	permConst_A;		/* Permeability constant (?). */
	double	permConst_B;		/* Units per second */
	double	releaseRate_g;		/* Release rate (units per second). */
	double	replenishRate_y;	/* Replenishment rate (units per second). */
	double	lossRate_l;			/* Loss rate (units per second). */
	double	reprocessRate_x;	/* Reprocessing rate (units per second). */
	double	recoveryRate_r;		/* Recovery rate (units per second). */
	double	maxFreePool_M;		/* Max. no. of transm. packets in free pool */
	double	firingRate_h;		/* Firing rate (spikes per second). */
	
} HairCell;

typedef struct {

		double	cleftC;			/* Quantity of transmitter in the cleft */
		double	reservoirQ;		/* Quantity of transmitter in the free pool */
		double	reprocessedW;	/* Quantity of transm. in the reproc. store */

} HairCellVars;
	
/******************************************************************************/
/*************************** External Variables *******************************/
/******************************************************************************/

extern	HairCell		hairCell;

/******************************************************************************/
/*************************** Function Prototypes ******************************/
/******************************************************************************/

BOOLEAN	CheckData_IHC_Meddis86(EarObjectPtr data);

BOOLEAN	CheckPars_IHC_Meddis86(void);

void	InitSpontOut_IHC_Meddis86(EarObjectPtr data, HairCellVars hCChannels[]);

void	PrintPars_IHC_Meddis86(void);

BOOLEAN	ReadPars_IHC_Meddis86(char *fileName);

BOOLEAN	RunModel_IHC_Meddis86(EarObjectPtr data);

BOOLEAN	SetFiringRate_IHC_Meddis86(double theSetFiringRate);

BOOLEAN	SetLossRate_IHC_Meddis86(double theLossRate);

BOOLEAN	SetMaxFreePool_IHC_Meddis86(double theMaxFreePool);

BOOLEAN	SetPars_IHC_Meddis86(double A, double B, double g, double y, double l,
		  double x, double r, double M, double h);
		  
BOOLEAN	SetPermConstA_IHC_Meddis86(double thePermConstA);

BOOLEAN	SetPermConstB_IHC_Meddis86(double thePermConstB);

BOOLEAN	SetReleaseRate_IHC_Meddis86(double theReleaseRate);

BOOLEAN	SetReplenishRate_IHC_Meddis86(double theReplenishRate);

BOOLEAN	SetReprocessRate_IHC_Meddis86(double theReprocessRate);

BOOLEAN	SetRecoveryRate_IHC_Meddis86(double theRecoveryRate);

#endif
