/******************
 *
 * MoMcGregor.h
 *
 * This is the header file for the module which implements M.J.Hewitt's version
 * of the MacGregor Model.
 * The changes implemented in "Neural Modeling of instrinsic and spike-discharge
 * properties of cochlear nucleus neurons", J. E. Erle and D. O. Kim,
 * Biological Cybernetics, Springer Verlag 1991. - have been added to this
 * module version of the McGregor model.
 *
 * It was revised from code written by Trevor Shackleton.
 *
 ******************/

#ifndef	_MOMCGREGOR_H
#define _MOMCGREGOR_H	1

/******************************************************************************/
/*************************** Constant Definitions *****************************/
/******************************************************************************/

#define	MCGREGOR_DE_LIMIT	0.7	/* This limits the value of G/tm => dE. */

/*
 * ns = nano siemens - unit of conductance.
 * The default values are for the Stellate model.
 */

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/

typedef struct  {

	BOOLEAN	membraneTConstFlag, thresholdTConstFlag, accomConstFlag;
	BOOLEAN	delayedRectKCondFlag, restingThresholdFlag, actionPotentialFlag;
	BOOLEAN	kDecayTConstFlag, kEquilibriumPotFlag, cellRestingPotFlag;

	double	membraneTConst_Tm;	/* Membrane time constant (s). */	
	double	kDecayTConst_TGk;	/* Potassium conductance decay constant (s). */
	double	thresholdTConst_TTh;/* Threshold rise time constant (s). */
	double	accomConst_c;		/* Accomdation constant (dimensionless). */
	double	delayedRectKCond_b;	/* Delayed Rectifier K Cond. strength (ns). */
	double	restingThreshold_Th0;/* Resting threshold of cell (mV). */
	double	actionPotential;	/* Action potential of spike (mv). */
	double	kEquilibriumPot_Ek;	/* Reversal pot. of the K conductance (mV). */
	double	cellRestingPot_Er;	/* Resting potential for K conductance (mV). */

} McGregor;

/******************************************************************************/
/*************************** External Variables *******************************/
/******************************************************************************/

extern	McGregor		mcGregor;

/******************************************************************************/
/*************************** Function Prototypes ******************************/
/******************************************************************************/

BOOLEAN		CheckPars_Neuron_McGregor(void);

double		GetPotentialResponse_Neuron_McGregor(double potential);

void		PrintPars_Neuron_McGregor(void);

BOOLEAN		ReadPars_Neuron_McGregor(char *fileName);

BOOLEAN		RunModel_Neuron_McGregor(EarObjectPtr data);

BOOLEAN		SetAccomConst_Neuron_McGregor(double theAccomConst);

BOOLEAN		SetActionPotential_Neuron_McGregor(double theActionPotential);

BOOLEAN		SetDelayedRectKCond_Neuron_McGregor(double theDelayedRectKCond);

BOOLEAN		SetKDecayTConst_Neuron_McGregor(double theKDecayTConst);

BOOLEAN		SetKEquilibriumPot_Neuron_McGregor(double theKEquilibriumPot);

BOOLEAN		SetCellRestingPot_Neuron_McGregor(double theCellRestingPot);

BOOLEAN		SetMembraneTConst_Neuron_McGregor(double theMembraneTConst);

BOOLEAN		SetPars_Neuron_McGregor(double tm, double tGk, double tTh,
			  double c, double b, double tH0, double aP, double ek, double er);

BOOLEAN		SetRestingThreshold_Neuron_McGregor(double theRestingThreshold);

BOOLEAN		SetThresholdTConst_Neuron_McGregor(double theThresholdTConst);

#endif
