/******************
 *
 * MoMcGregor2.h
 *
 * This is the header file for the module which implements M.J.Hewitt's version
 * of the MacGregor Model.
 * The changes implemented in "Neural Modeling of instrinsic and spike-discharge
 * properties of cochlear nucleus neurons", J. E. Erle and D. O. Kim,
 * Biological Cybernetics, Springer Verlag 1991. - have been added to this
 * module version of the McGregor model.
 *
 * It was revised from code written by Trevor Shackleton.
 *
 ******************/

#ifndef	_MOMCGREGOR2_H
#define _MOMCGREGOR2_H	1

/******************************************************************************/
/*************************** Constant Definitions *****************************/
/******************************************************************************/

#define	MCGREGOR2_DE_LIMIT	0.7	/* This limits the value of G/tm => dE. */

/*
 * ns = nano siemens - unit of conductance.
 * The default values are for the Stellate model.
 */

/******************************************************************************/
/*************************** Type definitions *********************************/
/******************************************************************************/

typedef struct  {

	BOOLEAN	membraneTConstFlag, thresholdTConstFlag, accomConstFlag;
	BOOLEAN	delayedRectKCondFlag, restingThresholdFlag, actionPotentialFlag;
	BOOLEAN	kDecayTConstFlag, nonLinearVConstFlag, kReversalPotenFlag;
	BOOLEAN	bReversalPotenFlag, kRestingCondFlag, bRestingCondFlag;

	double	membraneTConst_Tm;	/* Membrane time constant (s). */	
	double	kDecayTConst_TGk;	/* Potassium conductance decay constant (s). */
	double	thresholdTConst_TTh;/* Threshold rise time constant (s). */
	double	accomConst_c;		/* Accomdation constant (dimensionless). */
	double	delayedRectKCond_b;	/* Delayed Rectifier K Cond. strength (ns). */
	double	restingThreshold_Th0;/* Resting threshold of cell (mV). */
	double	actionPotential;	/* Action potential of spike (mv). */
	double	nonLinearVConst_Vnl;/* Non-linear voltage constant (mV). */
	double	kReversalPoten_Ek0;	/* Reversal pot. of the K conductance (mV). */
	double	bReversalPoten_Eb0;	/* Rev. pot. of all other conductances (mV) */
	double	kRestingCond_gk;	/* Resting potassium conductance (nS). */
	double	bRestingCond_gb;	/* Resting component for all other conds.(nS).*/	

} McGregor2;

/******************************************************************************/
/*************************** External Variables *******************************/
/******************************************************************************/

extern	McGregor2		mcGregor2;

/******************************************************************************/
/*************************** Function Prototypes ******************************/
/******************************************************************************/

BOOLEAN		CheckPars_Neuron_McGregor2(void);

double		GetPotentialResponse_Neuron_McGregor2(double potential);

void		PrintPars_Neuron_McGregor2(void);

BOOLEAN		ReadPars_Neuron_McGregor2(char *fileName);

BOOLEAN		RunModel_Neuron_McGregor2(EarObjectPtr data);

BOOLEAN		SetAccomConst_Neuron_McGregor2(double theAccomConst);

BOOLEAN		SetActionPotential_Neuron_McGregor2(double theActionPotential);

BOOLEAN		SetBRestingCond_Neuron_McGregor2(double theBRestingCond);

BOOLEAN		SetBReversalPoten_Neuron_McGregor2(double theBReversalPoten);

BOOLEAN		SetDelayedRectKCond_Neuron_McGregor2(double theDelayedRectKCond);

BOOLEAN		SetKDecayTConst_Neuron_McGregor2(double theKDecayTConst);

BOOLEAN		SetKRestingCond_Neuron_McGregor2(double theKRestingCond);

BOOLEAN		SetKReversalPoten_Neuron_McGregor2(double theKReversalPoten);

BOOLEAN		SetMembraneTConst_Neuron_McGregor2(double theMembraneTConst);

BOOLEAN		SetNonLinearVConst_Neuron_McGregor2(double theNonLinearVConst);

BOOLEAN		SetPars_Neuron_McGregor2(double tm, double tGk, double tTh,
			  double c, double b, double tH0, double aP, double vnl,
			  double ek0, double eb0, double gk, double gb);

BOOLEAN		SetRestingThreshold_Neuron_McGregor2(double theRestingThreshold);

BOOLEAN		SetThresholdTConst_Neuron_McGregor2(double theThresholdTConst);

#endif
