/*******
 *
 * MoRecptPotn.h
 *
 * This header file contains the model for the receptor potential.
 * It is planned to produce a module in the future which will encompass
 * This process stage and the Meddis93 hair cell model process stage in the
 * same model.  This will use less memory space.
 *
 * by Lowel O'Mard 18-2-93.
 *
 ********/

#ifndef	_MORECPTPOTN_H
#define _MORECPTPOTN_H	1
 
/******************************************************************************/ 
/*************************** Constant Definitions *****************************/ 
/******************************************************************************/ 

#define	MEDDIS93_q	0.5			/* For K+ probability: base Amplitude (uPa). */
#define	MEDDIS93_v	0.001		/* For K+ probability: amplitude scale (/uPa) */
#define	MEDDIS93_tm	0.4e-3		/* Basilar membrane time constant. */

/******************************************************************************/ 
/*************************** Type definitions *********************************/ 
/******************************************************************************/ 

typedef struct {

	BOOLEAN	paramQFlag, paramVFlag, bMTimeConstTmFlag;
	
	double	param_q;			/* For K+ probability: base Amplitude (uPa). */
	double	param_v;			/* For K+ probability: amplitude scale (/uPa) */
	double	bMTimeConst_tm;		/* Basilar membrane time constant. */
	
} RecptPotn;

/******************************************************************************/ 
/*************************** External Variables *******************************/ 
/******************************************************************************/ 

extern	RecptPotn	recptPotn;

/******************************************************************************/ 
/*************************** Function Prototypes ******************************/ 
/******************************************************************************/ 

BOOLEAN	CheckPars_IHC_RecptPotn(void);

void PrintPars_IHC_RecptPotn(void);

BOOLEAN	ReadPars_IHC_RecptPotn(char *fileName);

BOOLEAN	RunModel_IHC_RecptPotn(EarObjectPtr data);

BOOLEAN	SetBMTimeConstTm_IHC_RecptPotn(double theBMTimeConstTm);

BOOLEAN	SetParamQ_IHC_RecptPotn(double theParamQ);

BOOLEAN	SetParamV_IHC_RecptPotn(double theParamV);

BOOLEAN	SetPars_IHC_RecptPotn(double q, double v, double tm);
		  
#endif
