/*******
 *
 * UtDebug.c
 *
 * This module contains the debug routines used in testing.
 *
 * by Lowel O'Mard 26-10-93.
 *
 ********/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "LUTEar.h"
#include "SignalData.h"
#include "EarObject.h"
#include "UtDebug.h"

/******************************************************************************/
/*************************** Global Variables *********************************/
/******************************************************************************/

EarObjectPtr	debug = NULL;	/* This is used for debugging modules. */

/******************************************************************************/
/************************ Subroutines and functions ***************************/
/******************************************************************************/

/********************************* CheckInit **********************************/

/*
 * This routine checks that the gobal variable debug (hence the module) has
 * been initialised.  It prevents attempts to use uninitailised pointers which
 * will cause the module to crash the system.
 * It returns FALSE if the gobal variable has not been initialised.
 */
 
BOOLEAN
CheckInit_Debug(void)
{
	if (debug == NULL) {
		NotifyError("CheckInit_Debug: Debug module not initialised.");
		return(FALSE);
	}
	return(TRUE);
	
}

/*************************** Init *********************************************/

/*
 * This routine initialises the debug EarObject.
 */

void
Init_Debug(void)
{
	if (debug != NULL)
		Free_EarObject(&debug);
	debug = Init_EarObject("'debug' for Init_Debug");

}

/*************************** Init *********************************************/

/*
 * This routine frees the memory used by the debug EarObject.
 */

void
Free_Debug(void)
{
	if (debug != NULL)
		Free_EarObject(&debug);
		
}

/*************************** SetPars ******************************************/

/*
 * This function sets the debug EarObjects parameters.
 */
 
BOOLEAN
SetPars_Debug(int numChannels, double signalDuration, double samplingInterval)
{
	if (!CheckInit_Debug())
		return(FALSE);
	if (!InitOutSignal_EarObject(debug, numChannels, 
	  (ChanLen) (signalDuration / samplingInterval), samplingInterval)) {
		NotifyError("SetPars_Debug: Could not initialise output "\
		  "signal.");
		return(FALSE);
	}
	SetProcessName_EarObject(debug, "Debug module");
	return(TRUE);

}
