/**************
 *
 * UtERB.c
 *
 * Equivalent rectangular bandwidth (ERB) freqency functions.
 * These functions uses the Glasberg & Moore 1990 formulae.
 *
 * Unless otherwise stated all code was written by L. P. O'Mard.
 *
 *	E-mail: L.P.OMard@uk.ac.lut
 *
 * COPYRIGHT 
 *
 *	Speech and Hearing Laboratory,
 *	Department of Human Sciences,
 *	Loughborough University of Technology,
 *	Loughborough,
 *	Leicestershire LE11 3TU.
 *	United Kingdom.
 *
 *	12th July 1993.
 *
 *	DISCLAIMER OF WARRANTY
 *
 * Although every effort has been made to ensure the integrity of this code,
 * the author, Loughborough University and its Laboratories disclaim all
 * liability for direct or consequential damages resulting from its use.
 *
 **************/
 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "UtERB.h"

/******************************** ERBFromF ************************************/

/*
 * Calculate the erb at frequency f (new M&G formula).
 */

double
ERBFromF_ERB( double theFrequency )
{
	return ( K1_ERB * ( K2_ERB * theFrequency + 1.0 ) );

}

/******************************** FFromERB ************************************/

/*
 * Calculate the frequency at the associated ERB bandwidth (new M&G formula).
 */

double
FFromERB_ERB( double theERB)
{
	return ( (theERB / K1_ERB - 1.0) / K2_ERB );

}

/****************************** ERBRateFromF **********************************/

/*
 * Calculate the ERB rate (ERBs per Hz) at the specified frequency.
 * It assumes that no negative centre frequencies are used (They could cause
 * problems.
 */

double
ERBRateFromF_ERB(double theFrequency)
{
	return( K3_ERB * log10(K2_ERB * theFrequency + 1.0) );

}

/****************************** FFromERBRate **********************************/

/*
 * Calculate the frequency (ERBs per Hz) at the specified ERB rate.
 */

double
FFromERBRate_ERB(double theERBRate)
{
	return( (pow(10.0, theERBRate / K3_ERB) - 1.0) / K2_ERB);

}

